package com.xmt.gameet.controllers;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashSet;
import java.util.List;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpSession;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.security.access.annotation.Secured;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.ResponseBody;
import org.springframework.web.bind.annotation.SessionAttributes;
import org.springframework.web.servlet.ModelAndView;

import com.xmt.gameet.entities.Notification;
import com.xmt.gameet.entities.User;
import com.xmt.gameet.entities.UserRelationActual;
import com.xmt.gameet.helpers.EmailSender;
import com.xmt.gameet.services.GameService;
import com.xmt.gameet.services.NotificationService;
import com.xmt.gameet.services.UserService;

@PreAuthorize("hasRole('ROLE_USER')")
@RequestMapping("/events")
@SessionAttributes("authenticatedUser")
@Secured("ROLE_USER")
@Controller
public class EventController {
	
	@Autowired
	@Qualifier("userService")
	UserService userService;
	
	@Autowired
	@Qualifier("notificationService")
	NotificationService notificationService;
	
	@Autowired
	@Qualifier("gameService")
	GameService gameService;
	
	@RequestMapping("")
	public ModelAndView events(HttpSession session){
		ModelAndView mv = new ModelAndView("events");
		if(session.getAttribute("authenticatedUser") == null){
			mv = new ModelAndView("redirect:/welcome");
		}
		else{
			User authenticatedUser = (User) session.getAttribute("authenticatedUser");
			mv.addObject("profileComplete", authenticatedUser.isProfileCompleted());
			mv.addObject("authenticatedUser", authenticatedUser);
			mv.addObject("games",gameService.getAllGames());
		}
		return mv;
	}
	
	@RequestMapping(value = "/getEvents", method = RequestMethod.POST)
	@ResponseBody
	public List<Notification> getEvents(HttpSession session){
		User authenticatedUser = (User) session.getAttribute("authenticatedUser");
		List<Notification> userNotifications = new ArrayList<Notification>();
		for(Notification notif : authenticatedUser.getUserNotifications()){
			userNotifications.add(notif);
		}
		
		return userNotifications;
	}
	
	@RequestMapping(value = "/getPendingEvents", method = RequestMethod.POST)
	@ResponseBody
	public List<Notification> getPendingEvents(HttpSession session){
		User authenticatedUser = (User) session.getAttribute("authenticatedUser");
		List<Notification> userNotifications = notificationService.getNotificationsPending(authenticatedUser.getId());
		
		return userNotifications;
	}
	
	@RequestMapping(value = "/eventInvite", method = RequestMethod.POST)
	@ResponseBody
	public String sendFriendInvite(@RequestParam("userId") Integer userId,@RequestParam("date") String date,
			@RequestParam("gameId") Integer gameId, @RequestParam("hour") String hour,
			HttpSession session, Model model, HttpServletRequest request){
		User authenticatedUser = (User) session.getAttribute("authenticatedUser");
		User user = userService.getUser(userId);
		if(authenticatedUser.getUserNotifications() == null){
			authenticatedUser.setUserNotifications(new HashSet<Notification>());
		}
		if(user != null){
			Notification notif = new Notification();
			notif.setStatus(1);
			notif.setUser1(authenticatedUser);
			notif.setUser2(user);
			SimpleDateFormat sdf = new SimpleDateFormat("dd/MM/yyyy");
			Date eventDate;
			try {
				eventDate = sdf.parse(date);
				notif.setDay(eventDate);
			} catch (ParseException e) {
				System.out.println("Error al parsear la fecha. " + e.getStackTrace());
				e.printStackTrace();
			}

			notif.setHour(hour);
			notif.setGame(gameService.getGame(gameId));
			notif = notificationService.create(notif);
			authenticatedUser.getUserNotifications().add(notif);
			if(user.getNotificationsEvents()){
				EmailSender.sendEmail(EmailSender.EVENT_INVITE, user.getEmail(), EmailSender.eventRequest(authenticatedUser, user, request.getRequestURL().toString().replace("/events/eventInvite","")));
			}
			model.addAttribute("authenticatedUser", authenticatedUser);
			return "success";
		}
		else{
			return "failure";
		}
	}
	
	@RequestMapping(value = "/acceptEvent", method = RequestMethod.POST)
	@ResponseBody
	public String acceptEvent(@RequestParam("eventId") Integer eventId,
			HttpSession session, Model model){
		User authenticatedUser = (User) session.getAttribute("authenticatedUser");
		Notification notification = notificationService.getNotification(eventId);
		if(authenticatedUser.getUserNotifications() == null){
			authenticatedUser.setUserNotifications(new HashSet<Notification>());
		}
		if(notification != null){
			notification.setStatus(2);
			notification = notificationService.update(notification);
			User user = userService.getUser(notification.getUser1().getId());
			Notification inverseNotification = new Notification();
			inverseNotification.setHour(notification.getHour());
			inverseNotification.setStatus(2);
			inverseNotification.setDay(notification.getDay());
			inverseNotification.setGame(notification.getGame());
			inverseNotification.setUser1(authenticatedUser);
			inverseNotification.setUser2(user);
			inverseNotification = notificationService.create(inverseNotification);
			authenticatedUser.getUserNotifications().add(inverseNotification);
			model.addAttribute("authenticatedUser", authenticatedUser);
			return "success";
		}
		else{
			return "failure";
		}
	}
	
	@RequestMapping(value = "/rejectEvent", method = RequestMethod.POST)
	@ResponseBody
	public String rejectFriend(@RequestParam("eventId") Integer eventId,
			HttpSession session, Model model){
		User authenticatedUser = (User) session.getAttribute("authenticatedUser");
		Notification notification = notificationService.getNotification(eventId);
		if(authenticatedUser.getUserRelations() == null){
			authenticatedUser.setUserRelations(new HashSet<UserRelationActual>());
		}
		if(notification != null){
			notification.setStatus(3);
			notification = notificationService.update(notification);
			return "success";
		}
		else{
			return "failure";
		}
	}

}
