package com.xmt.gameet.controllers;

import java.util.Map;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.security.core.Authentication;
import org.springframework.security.core.context.SecurityContextHolder;
import org.springframework.security.core.userdetails.UserDetails;
import org.springframework.security.web.authentication.logout.SecurityContextLogoutHandler;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.ResponseBody;
import org.springframework.web.bind.annotation.SessionAttributes;
import org.springframework.web.servlet.ModelAndView;
import org.springframework.web.servlet.mvc.support.RedirectAttributes;
import org.springframework.web.servlet.view.RedirectView;

import com.xmt.gameet.entities.User;
import com.xmt.gameet.helpers.EmailSender;
import com.xmt.gameet.helpers.ErrorMessages;
import com.xmt.gameet.helpers.Utils;
import com.xmt.gameet.services.UserService;

@RequestMapping()
@Controller
@SessionAttributes("authenticatedUser")
public class LoginController {
	
	@Autowired
	@Qualifier("userService")
	UserService userService;
	
	@RequestMapping("/welcome")
	public ModelAndView welcome(Model model){
		ModelAndView mv = new ModelAndView("welcome");
		mv.addObject("registrationForm", new User());
		mv.addObject("authenticatedUser", new User());
		return mv;
	}
	
	@RequestMapping("")
	public ModelAndView index(Model model){
		ModelAndView mv = null;
		try{
			Authentication auth = SecurityContextHolder.getContext().getAuthentication();
			UserDetails user = (UserDetails) auth.getPrincipal();
			User authenticatedUser = userService.findUserByEmail(user.getUsername());
			System.out.println("Usuario logueado: " + authenticatedUser.getEmail() + ", completado? " + authenticatedUser.isProfileCompleted());
			if(authenticatedUser.isProfileCompleted()){
				mv = new ModelAndView("redirect:search");
			}
			else{
				mv = new ModelAndView("redirect:user/profile");
			}
			model.addAttribute("authenticatedUser", authenticatedUser);
		} catch(Exception ex){
			mv = new ModelAndView("redirect:welcome");
		}
		return mv;
	}
	
	@RequestMapping("/login")
	public ModelAndView login(@RequestParam(value="error", required=false, defaultValue="true") Boolean error){
		ModelAndView mv = null;
		System.out.println(error.toString());
		if(error != null && error){
			mv = new ModelAndView("redirect:welcome");
		}else{
			mv = new ModelAndView("redirect:search");
		}
		return mv;
	}
	
	@RequestMapping(value="/logout", method = RequestMethod.GET)
	public String logoutPage (HttpServletRequest request, HttpServletResponse response) {
	    Authentication auth = SecurityContextHolder.getContext().getAuthentication();
	    if (auth != null){    
	        new SecurityContextLogoutHandler().logout(request, response, auth);
	    }
	    return "redirect:/login?logout";
	}
	
	@RequestMapping("/register")
	public ModelAndView registerUser(@ModelAttribute("registrationForm") User user, BindingResult result, HttpServletRequest request,
			RedirectAttributes ra){
		RedirectView view = new RedirectView("/welcome",true);
		view.setExposeModelAttributes(false);
		ModelAndView mv = new ModelAndView(view);
		Map<String,String> errors = userService.verifyFirstRegistration(user);
		if(errors.size() > 0){
			for(String key : errors.keySet()){
				ra.addFlashAttribute(key, errors.get(key));
			}
		}
		else{
			try{
				userService.registerUser(user, request.getRequestURL().toString().replace("/register",""));
				ra.addFlashAttribute("registration", "success");
			} catch(Exception ex){
				System.out.println(ex);
				ra.addFlashAttribute("emailError", ErrorMessages.INTERNAL_ERROR);
			}
		}
		ra.addFlashAttribute("registrationForm", user);
		return mv;
	}
	
	@RequestMapping("/confirmEmail")
	public ModelAndView confirmEmail(@RequestParam("id") Integer id){
		ModelAndView mv = new ModelAndView("welcome");
		User user = new User();
		user.setId(id);
		boolean result = userService.confirmUserEmail(user);
		mv.addObject("emailConfirmed", result);
		mv.addObject("registrationForm", new User());
		return mv;
	}
	
	@RequestMapping("/sendPasswordRecovery")
	public ModelAndView sendPasswordRecovery(@RequestParam("email") String email, HttpServletRequest request,RedirectAttributes ra){
		RedirectView view = new RedirectView("/welcome",true);
		view.setExposeModelAttributes(false);
		ModelAndView mv = new ModelAndView(view);
		User user = userService.findUserByEmail(email);
		if(user == null){
			ra.addFlashAttribute("emailNotUsed", "El correo electrnico especificado no pertenece a ningn usuario");
		}
		else{
			EmailSender.sendEmail(EmailSender.PASSWORD_RECOVERY, user.getEmail(), EmailSender.passwordRecovery(user, request.getRequestURL().toString().replace("/sendPasswordRecovery","")));
			ra.addFlashAttribute("mailSend", true);
		}
		return mv;
	}
	
	@RequestMapping("/passwordRecovery")
	public ModelAndView passwordRecovery(@RequestParam("userId") String userId, HttpServletRequest request,RedirectAttributes ra){
		RedirectView view = new RedirectView("/welcome",true);
		view.setExposeModelAttributes(false);
		ModelAndView mv = new ModelAndView(view);
		ra.addFlashAttribute("userId", userId);
		ra.addFlashAttribute("passwordRecovery", true);
		return mv;
	}
	
	@RequestMapping(value = "/changePassword", method = RequestMethod.POST)
	@ResponseBody
	public String changePassword(@RequestParam("userId") Integer userId,
			@RequestParam("password") String password,
			@RequestParam("repeatPassword") String repeatPassword,
			HttpServletRequest request){
		User user = userService.getUser(userId);
		user.setNewPassword(password);
		user.setRepeatedPassword(repeatPassword);
		Map<String,String> errors = userService.verifyPasswords(user);
		if(errors.keySet().size() > 0){
			return "fail";
		}
		else{
			user.setPassword(Utils.getEncryptedPassword(user.getNewPassword()));
			userService.updateUser(user);
		}
		return "success";
	}
	
	

}
