package com.xmt.gameet.controllers;

import java.util.Date;
import java.util.HashSet;
import java.util.Set;

import javax.servlet.http.HttpSession;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.security.access.annotation.Secured;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.ResponseBody;
import org.springframework.web.bind.annotation.SessionAttributes;
import org.springframework.web.servlet.ModelAndView;
import org.springframework.web.servlet.mvc.support.RedirectAttributes;
import org.springframework.web.servlet.view.RedirectView;

import com.xmt.gameet.entities.MessageMail;
import com.xmt.gameet.entities.User;
import com.xmt.gameet.helpers.ErrorMessages;
import com.xmt.gameet.services.MessagingService;
import com.xmt.gameet.services.UserService;

@PreAuthorize("hasRole('ROLE_USER')")
@RequestMapping("/inbox")
@SessionAttributes("authenticatedUser")
@Secured("ROLE_USER")
@Controller
public class MessagesController {
	
	@Autowired
	@Qualifier("userService")
	UserService userService;
	
	@Autowired
	@Qualifier("messagingService")
	MessagingService messageService;
	

	
	@RequestMapping("")
	public ModelAndView inbox(HttpSession session){
		ModelAndView mv = new ModelAndView("messages");
		if(session.getAttribute("authenticatedUser") == null){
			mv = new ModelAndView("redirect:/welcome");
		}
		else{
			User authenticatedUser = (User) session.getAttribute("authenticatedUser");
			mv.addObject("profileComplete", authenticatedUser.isProfileCompleted());
			mv.addObject("authenticatedUser", authenticatedUser);
			MessageMail message = new MessageMail();
			message.setUser2(new User());
			if(session.getAttribute("newMessage") == null){
				mv.addObject("newMessage", message);
			}
			mv.addObject("noMessages", authenticatedUser.getMessagesReceived().size() == 0);
			if(authenticatedUser.getMessagesReceived() != null){
				mv.addObject("messageReceived",authenticatedUser.getMessagesReceived().toArray()[0]);
			}
			else{
				mv.addObject("messageReceived",null);
			}
		}
		return mv;
	}
	
	@RequestMapping("/newMessage")
	public ModelAndView newMessage(@RequestParam("userId") Integer userId,HttpSession session){
		ModelAndView mv = new ModelAndView("messages");
		if(session.getAttribute("authenticatedUser") == null){
			mv = new ModelAndView("redirect:/welcome");
		}
		else{
			User authenticatedUser = (User) session.getAttribute("authenticatedUser");
			mv.addObject("profileComplete", authenticatedUser.isProfileCompleted());
			mv.addObject("authenticatedUser", authenticatedUser);
			MessageMail message = new MessageMail();
			message.setUser2(new User());
			mv.addObject("newMessage", message);
			mv.addObject("noMessages", authenticatedUser.getMessagesReceived().size() == 0);
			mv.addObject("isNewMessage", true);
			User user = userService.getUser(userId);
			mv.addObject("toThisUser", user.getNickname());
			if(authenticatedUser.getMessagesReceived() != null){
				mv.addObject("messageReceived",authenticatedUser.getMessagesReceived().toArray()[0]);
			}
			else{
				mv.addObject("messageReceived",null);
			}
		}
		return mv;
	}
	
	@RequestMapping("/getMessage")
	public @ResponseBody MessageMail getMessage(@RequestParam("id") Integer id, HttpSession session){
		User authenticatedUser = (User) session.getAttribute("authenticatedUser");
		MessageMail messageSelected = null;
		for(MessageMail mail : authenticatedUser.getMessagesReceived()){
			if(mail.getId() == id){
				messageSelected = mail;
			}
		}
		if(messageSelected == null){
			for(MessageMail mail : authenticatedUser.getMessagesSent()){
				if(mail.getId() == id){
					messageSelected = mail;
				}
			}
		}
		return messageSelected;
	}
	
	@RequestMapping("/deleteMessage")
	public ModelAndView deleteMessage(@RequestParam("id") Integer id, HttpSession session){
		RedirectView view = new RedirectView("/inbox",true);
		ModelAndView mv = new ModelAndView(view);
		User authenticatedUser = (User) session.getAttribute("authenticatedUser");
		MessageMail messageToDelete = null;
		for(MessageMail mail : authenticatedUser.getMessagesReceived()){
			if(mail.getId() == id && mail.getUser2() == authenticatedUser.getId()){
				messageToDelete = mail;
				messageToDelete.setUser2Deleted(true);
			}
		}
		for(MessageMail mail : authenticatedUser.getMessagesSent()){
			if(mail.getId() == id && mail.getUser1() == authenticatedUser.getId()){
				messageToDelete = mail;
				messageToDelete.setUser1Deleted(true);
			}
		}
		try{
			messageService.updateMessageMail(messageToDelete);
		} catch(Exception e){
			System.out.println("Error eliminando mensaje: " + e);
		}
		return mv;
	}
	
	@RequestMapping("/sendMessage")
	public ModelAndView sendMessage(@ModelAttribute("newMessage") MessageMail message, RedirectAttributes ra ,HttpSession session){
		RedirectView view = new RedirectView("/inbox",true);
		ModelAndView mv = new ModelAndView(view);
		if(session.getAttribute("authenticatedUser") == null){
			mv = new ModelAndView("redirect:/welcome");
		}
		else{
			User authenticatedUser = (User) session.getAttribute("authenticatedUser");
			User sendTo = userService.getUserByNickname(message.getToUserNameProp());
			System.out.println("Titulo: " + message.getTitle());
			System.out.println("Usuario: " + message.getToUserNameProp());
			System.out.println("Contenido: " + message.getMessage());
			boolean error = false;
			if(sendTo == null){
				ra.addFlashAttribute("userNameError",ErrorMessages.MAIL_USERNAME_ERROR);
				error = true;
			}
			if(message.getTitle() == null || message.getTitle().isEmpty()){
				ra.addFlashAttribute("titleError",ErrorMessages.MAIL_TITLE_EMPTY);
				error = true;
			}
			if(message.getMessage() == null || message.getMessage().isEmpty()){
				ra.addFlashAttribute("contentError",ErrorMessages.MAIL_CONTENT_ERROR);
				error = true;
			}
			if(!error){
				message.setSent(new Date());
				message.setUser1(authenticatedUser);
				message.setUser2(sendTo);
				message.setGlobal(false);
				message.setRead(false);
				authenticatedUser.getMessagesSent().add(message);
				messageService.createMessageMail(message);
				session.setAttribute("authenticatedUser", authenticatedUser);
			}
			else{
				ra.addFlashAttribute("newMessage",message);
				ra.addFlashAttribute("isNewMessage", true);
				ra.addFlashAttribute("toThisUser",message.getToUserNameProp());
			}
		}
		return mv;
	}

	
}
