package com.xmt.gameet.controllers;

import javax.servlet.http.HttpSession;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.security.access.annotation.Secured;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.stereotype.Controller;
import org.springframework.ui.ModelMap;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.SessionAttributes;
import org.springframework.web.servlet.ModelAndView;

import com.xmt.gameet.entities.User;
import com.xmt.gameet.helpers.FormHelper;
import com.xmt.gameet.services.CountryService;
import com.xmt.gameet.services.GameService;
import com.xmt.gameet.services.GenreService;
import com.xmt.gameet.services.LanguageService;
import com.xmt.gameet.services.PlatformService;
import com.xmt.gameet.services.UserService;

@PreAuthorize("hasRole('ROLE_USER')")
@RequestMapping("")
@SessionAttributes("authenticatedUser")
@Secured("ROLE_USER")
@Controller
public class SearchController {
	
	@Autowired
	@Qualifier("userService")
	UserService userService;
	
	@Autowired
	@Qualifier("countryService")
	CountryService countryService;
	
	@Autowired
	@Qualifier("genreService")
	GenreService genreService;
	
	@Autowired
	@Qualifier("platformService")
	PlatformService platformService;
	
	@Autowired
	@Qualifier("gameService")
	GameService gameService;
	
	@Autowired
	@Qualifier("languageService")
	LanguageService languageService;

	
	@RequestMapping("/search")
	public ModelAndView index(HttpSession session){
		ModelAndView mv = new ModelAndView("search");
		loadModelAndViewItems(mv);
		if(session.getAttribute("authenticatedUser") == null){
			mv = new ModelAndView("redirect:/welcome");
		}
		else{
			User authenticatedUser = (User) session.getAttribute("authenticatedUser");
			mv.addObject("profileComplete", authenticatedUser.isProfileCompleted());
			mv.addObject("authenticatedUser", authenticatedUser);
		}
		return mv;
	}
	
	@RequestMapping("/testSearch")
	public ModelAndView testSearch(@ModelAttribute("SpringWeb")User user, ModelMap model){
		ModelAndView mv = new ModelAndView("index");
		return mv;
	}
	
	private void loadModelAndViewItems(ModelAndView mv){
		mv.addObject("countries", countryService.getAllCountries());
		mv.addObject("genres", genreService.getAllGenre());
		mv.addObject("platforms", platformService.getAllPlatforms());
		mv.addObject("languages",languageService.getAllLanguage());
		mv.addObject("games",gameService.getAllGames());
		mv.addObject("days", FormHelper.daysOfTheWeek());
	}

}
