package com.xmt.gameet.controllers;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

import javax.servlet.ServletOutputStream;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;
import javax.validation.Valid;

import org.apache.commons.io.IOUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.security.access.annotation.Secured;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.ResponseBody;
import org.springframework.web.bind.annotation.SessionAttributes;
import org.springframework.web.servlet.ModelAndView;
import org.springframework.web.servlet.mvc.support.RedirectAttributes;
import org.springframework.web.servlet.view.RedirectView;

import com.xmt.gameet.entities.Game;
import com.xmt.gameet.entities.Genre;
import com.xmt.gameet.entities.Language;
import com.xmt.gameet.entities.Platform;
import com.xmt.gameet.entities.RecomendationParameter;
import com.xmt.gameet.entities.RequestStatus;
import com.xmt.gameet.entities.Timetable;
import com.xmt.gameet.entities.User;
import com.xmt.gameet.entities.UserRelationActual;
import com.xmt.gameet.entities.UserRelationHistoric;
import com.xmt.gameet.helpers.EmailSender;
import com.xmt.gameet.helpers.Utils;
import com.xmt.gameet.services.CountryService;
import com.xmt.gameet.services.GameService;
import com.xmt.gameet.services.GenreService;
import com.xmt.gameet.services.LanguageService;
import com.xmt.gameet.services.PlatformService;
import com.xmt.gameet.services.RecomendationParameterService;
import com.xmt.gameet.services.RequestStatusService;
import com.xmt.gameet.services.TimetableService;
import com.xmt.gameet.services.UserLoginService;
import com.xmt.gameet.services.UserService;

@RequestMapping("/user")
@Controller
@SessionAttributes("authenticatedUser")
@Secured("ROLE_USER")
public class UserController {
	
	@Autowired
	@Qualifier("userLoginService")
	UserLoginService userLoginService;
	
	@Autowired
	@Qualifier("userService")
	UserService userService;
	
	@Autowired
	@Qualifier("countryService")
	CountryService countryService;
	
	@Autowired
	@Qualifier("genreService")
	GenreService genreService;
	
	@Autowired
	@Qualifier("platformService")
	PlatformService platformService;
	
	@Autowired
	@Qualifier("gameService")
	GameService gameService;
	
	@Autowired
	@Qualifier("languageService")
	LanguageService languageService;
	
	@Autowired
	@Qualifier("timetableService")
	TimetableService timetableService;
	
	@Autowired
	@Qualifier("requestStatusService")
	RequestStatusService requestStatusService;
	
	@Autowired
	@Qualifier("recomendationParameterService")
	RecomendationParameterService recomendationParamaterService;

	@RequestMapping("/profile")
	public ModelAndView userProfile(HttpSession session, Model model){
		ModelAndView mv = new ModelAndView("profile");
		loadModelAndViewItems(mv);
		if(session.getAttribute("authenticatedUser") == null){
			mv = new ModelAndView("redirect:/welcome");
		}
		else{
			User authenticatedUser = (User) session.getAttribute("authenticatedUser");
			mv.addObject("profileComplete", authenticatedUser.isProfileCompleted());
			mv.addObject("authenticatedUser", authenticatedUser);
		}
		return mv;
	}
	
	@RequestMapping("/save")
	public ModelAndView saveUserInformation(@Valid @ModelAttribute("authenticatedUser") User authenticatedUser,
			final BindingResult result,
			RedirectAttributes ra, Model model){
		RedirectView view = new RedirectView("/user/profile",true);
		view.setExposeModelAttributes(false);
		ModelAndView mv = new ModelAndView(view);
		Map<String,String> errors = null;
		if(authenticatedUser.getNewPassword() != null && !authenticatedUser.getNewPassword().isEmpty()){
			errors = userService.verifyPasswords(authenticatedUser);
			for(String key : errors.keySet()){
				ra.addFlashAttribute(key, errors.get(key));
			}
		}
		if(!result.hasErrors() && errors == null){
			if(authenticatedUser.getNewPassword() != null && !authenticatedUser.getNewPassword().isEmpty()){
				authenticatedUser.setPassword(Utils.getEncryptedPassword(authenticatedUser.getNewPassword()));
			}
			ra.addFlashAttribute("success", true);
			authenticatedUser = userService.updateUser(authenticatedUser);
			model.addAttribute("authenticatedUser", authenticatedUser);
		}
		else{
			ra.addFlashAttribute("org.springframework.validation.BindingResult.authenticatedUser", result);
		}
		ra.addFlashAttribute("profileComplete", authenticatedUser.isProfileCompleted());
		ra.addFlashAttribute("authenticatedUser", authenticatedUser);
		return mv;
	}
	
	@RequestMapping(value="/addGame", method = RequestMethod.POST, produces = "application/json")
	public @ResponseBody String addGameToUser(@RequestParam("idGame") Integer idGame,HttpSession session,Model model){
		String result = "";
		if(session.getAttribute("authenticatedUser") == null){
			result = "failure";
		}
		else{
			User authenticatedUser = (User) session.getAttribute("authenticatedUser");
			Game game = gameService.getGame(idGame);
			if(game != null){
				authenticatedUser = userService.getUser(authenticatedUser.getId());
				if(authenticatedUser.getGamesToPlay() == null){
					authenticatedUser.setGamesToPlay(new HashSet<Game>());
				}
				authenticatedUser.getGamesToPlay().add(game);
				authenticatedUser = userService.updateUser(authenticatedUser);
				model.addAttribute("authenticatedUser", authenticatedUser);
				result = "success";
			}
			else{
				result = "failure";
			}
		}
		return result;
	}
	
	@RequestMapping(value="/removeGame", method = RequestMethod.POST, produces = "application/json")
	public @ResponseBody String removeGameFromUser(@RequestParam("idGame") Integer idGame,HttpSession session, Model model){
		String result = "";
		if(session.getAttribute("authenticatedUser") == null){
			result = "failure";
		}
		else{
			User authenticatedUser = (User) session.getAttribute("authenticatedUser");
			if(authenticatedUser.getGamesToPlay() != null){
				Set<Game> newGameList = new HashSet<Game>();
				for(Game game : authenticatedUser.getGamesToPlay()){
					if(game.getId() != idGame){
						newGameList.add(game);
					}
				}
				authenticatedUser.setGamesToPlay(newGameList);
				authenticatedUser = userService.updateUser(authenticatedUser);
				model.addAttribute("authenticatedUser", authenticatedUser);
				result = "success";
			}
			else{
				result = "failure";
			}
		}
		return result;
	}
	
	@RequestMapping(value="/addGenre", method = RequestMethod.POST, produces = "application/json")
	public @ResponseBody String addGenreToUser(@RequestParam("idGenre") Integer idGenre,HttpSession session, Model model){
		String result = "";
		if(session.getAttribute("authenticatedUser") == null){
			result = "failure";
		}
		else{
			User authenticatedUser = (User) session.getAttribute("authenticatedUser");
			Genre genre = genreService.getGenre(idGenre);
			if(genre != null){
				authenticatedUser = userService.getUser(authenticatedUser.getId());
				if(authenticatedUser.getGenresPrefered() == null){
					authenticatedUser.setGenresPrefered(new HashSet<Genre>());
				}
				authenticatedUser.getGenresPrefered().add(genre);
				authenticatedUser = userService.updateUser(authenticatedUser);
				model.addAttribute("authenticatedUser", authenticatedUser);
				result = "success";
			}
			else{
				result = "failure";
			}
		}
		return result;
	}
	
	@RequestMapping(value="/removeGenre", method = RequestMethod.POST, produces = "application/json")
	public @ResponseBody String removeGenreFromUser(@RequestParam("idGenre") Integer idGenre,HttpSession session){
		String result = "";
		if(session.getAttribute("authenticatedUser") == null){
			result = "failure";
		}
		else{
			User authenticatedUser = (User) session.getAttribute("authenticatedUser");
			if(authenticatedUser.getGamesToPlay() != null){
				Set<Genre> newGenreList = new HashSet<Genre>();
				for(Genre genre : authenticatedUser.getGenresPrefered()){
					if(genre.getId() != idGenre){
						newGenreList.add(genre);
					}
				}
				authenticatedUser.setGenresPrefered(newGenreList);
				userService.updateUser(authenticatedUser);
				result = "success";
			}
			else{
				result = "failure";
			}
		}
		return result;
	}
	
	@RequestMapping(value="/addPlatform", method = RequestMethod.POST, produces = "application/json")
	public @ResponseBody String addPlatformToUser(@RequestParam("idPlatform") Integer idPlatform,HttpSession session){
		String result = "";
		if(session.getAttribute("authenticatedUser") == null){
			result = "failure";
		}
		else{
			User authenticatedUser = (User) session.getAttribute("authenticatedUser");
			Platform platform = platformService.getPlatform(idPlatform);
			if(platform != null){
				authenticatedUser = userService.getUser(authenticatedUser.getId());
				if(authenticatedUser.getPlatformsPrefered() == null){
					authenticatedUser.setPlatformsPrefered(new HashSet<Platform>());
				}
				authenticatedUser.getPlatformsPrefered().add(platform);
				userService.updateUser(authenticatedUser);
				result = "success";
			}
			else{
				result = "failure";
			}
		}
		return result;
	}
	
	@RequestMapping(value="/removePlatform", method = RequestMethod.POST, produces = "application/json")
	public @ResponseBody String removePlatformFromUser(@RequestParam("idPlatform") Integer idPlatform,HttpSession session){
		String result = "";
		if(session.getAttribute("authenticatedUser") == null){
			result = "failure";
		}
		else{
			User authenticatedUser = (User) session.getAttribute("authenticatedUser");
			if(authenticatedUser.getPlatformsPrefered() != null){
				Set<Platform> newPlatformList = new HashSet<Platform>();
				for(Platform platform : authenticatedUser.getPlatformsPrefered()){
					if(platform.getId() != idPlatform){
						newPlatformList.add(platform);
					}
				}
				authenticatedUser.setPlatformsPrefered(newPlatformList);
				userService.updateUser(authenticatedUser);
				result = "success";
			}
			else{
				result = "failure";
			}
		}
		return result;
	}
	
	@RequestMapping(value="/addLanguage", method = RequestMethod.POST, produces = "application/json")
	public @ResponseBody String addLanguageToUser(@RequestParam("idLanguage") Integer idLanguage,HttpSession session){
		String result = "";
		if(session.getAttribute("authenticatedUser") == null){
			result = "failure";
		}
		else{
			User authenticatedUser = (User) session.getAttribute("authenticatedUser");
			Language language = languageService.getLanguage(idLanguage);
			if(language != null){
				authenticatedUser = userService.getUser(authenticatedUser.getId());
				if(authenticatedUser.getLanguages() == null){
					authenticatedUser.setLanguages(new HashSet<Language>());
				}
				authenticatedUser.getLanguages().add(language);
				userService.updateUser(authenticatedUser);
				result = "success";
			}
			else{
				result = "failure";
			}
		}
		return result;
	}
	
	@RequestMapping(value="/removeLanguage", method = RequestMethod.POST, produces = "application/json")
	public @ResponseBody String removeLanguageFromUser(@RequestParam("idLanguage") Integer idLanguage,HttpSession session){
		String result = "";
		if(session.getAttribute("authenticatedUser") == null){
			result = "failure";
		}
		else{
			User authenticatedUser = (User) session.getAttribute("authenticatedUser");
			if(authenticatedUser.getLanguages() != null){
				Set<Language> newLanguageList = new HashSet<Language>();
				for(Language language : authenticatedUser.getLanguages()){
					if(language.getId() != idLanguage){
						newLanguageList.add(language);
					}
				}
				authenticatedUser.setLanguages(newLanguageList);
				userService.updateUser(authenticatedUser);
				result = "success";
			}
			else{
				result = "failure";
			}
				
		}
		return result;
	}
	
	@RequestMapping("/saveNotificationConfig")
	public @ResponseBody String saveUserNotificationConfig(@RequestParam("friends") boolean friends,
			@RequestParam("news") boolean news,
			@RequestParam("messages") boolean messages,
			@RequestParam("events") boolean events,
			HttpSession session){
		if(session.getAttribute("authenticatedUser") == null){
			return "failure";
		}
		try{
			User authenticatedUser = (User) session.getAttribute("authenticatedUser");
			authenticatedUser.setNotificationsFriends(friends);
			authenticatedUser.setNotificationsNews(news);
			authenticatedUser.setNotificationsMessages(messages);
			authenticatedUser.setNotificationsEvents(events);
			userService.updateUser(authenticatedUser);
			return "success";
		}catch(Exception ex){
			return "failure";
		}
	}
	
	@RequestMapping(value = "/getTimetable", method = RequestMethod.POST)
	@ResponseBody
	public Set<Timetable> getUserTimetable(HttpSession session){
		User authenticatedUser = (User) session.getAttribute("authenticatedUser");
		return authenticatedUser.getTimetable();
	}
	
	@RequestMapping(value = "/saveTimetable", method = RequestMethod.POST)
	@ResponseBody
	public String saveUserTimetable(@RequestParam(value="mon[]", required=false) String[] monday,
			@RequestParam(value="tue[]",required=false) String[] tuesday,
			@RequestParam(value="wed[]",required=false) String[] wednesday,
			@RequestParam(value="thu[]",required=false) String[] thursday,
			@RequestParam(value="fri[]",required=false) String[] friday,
			@RequestParam(value="sat[]",required=false) String[] saturday,
			@RequestParam(value="sun[]",required=false) String[] sunday,
			HttpSession session, Model model) throws Exception{
		try{
			User authenticatedUser = (User) session.getAttribute("authenticatedUser");
			if(authenticatedUser.getTimetable() == null){
				authenticatedUser.setTimetable(new HashSet<Timetable>());
			}
			List<Timetable> timetables = new ArrayList<Timetable>();
			if(monday != null){
				for(int i = 0; i < monday.length ; i++){
					timetables.add(Utils.createTimetable(monday[i], "Mon"));
				}
			}
			if(tuesday != null){
				for(int i = 0; i < tuesday.length ; i++){
					timetables.add(Utils.createTimetable(tuesday[i], "Tue"));
				}
			}
			if(wednesday != null){
				for(int i = 0; i < wednesday.length ; i++){
					timetables.add(Utils.createTimetable(wednesday[i], "Wed"));
				}
			}
			if(thursday != null){
				for(int i = 0; i < thursday.length ; i++){
					timetables.add(Utils.createTimetable(thursday[i], "Thu"));
				}
			}
			if(friday != null){
				for(int i = 0; i < friday.length ; i++){
					timetables.add(Utils.createTimetable(friday[i], "Fri"));
				}
			}
			if(saturday != null){
				for(int i = 0; i < saturday.length ; i++){
					timetables.add(Utils.createTimetable(saturday[i], "Sat"));
				}
			}
			if(sunday != null){
				for(int i = 0; i < sunday.length ; i++){
					timetables.add(Utils.createTimetable(sunday[i], "Sun"));
				}
			}
			timetableService.deleteAllByUser(authenticatedUser);
			System.out.println("HORARIOS: " + timetables.size());
			for(Timetable timetable : timetables){
				timetable.setUser(authenticatedUser);
				timetableService.create(timetable);
			}
			authenticatedUser = userService.getUser(authenticatedUser.getId());
			model.addAttribute("authenticatedUser", authenticatedUser);
		}catch (Exception ex){
			throw new Exception(ex);
			//System.out.println("Error al guardar el horario: " + ex.getMessage());
			//return "failure";
		}
		
		return "success";
	}
	
	@RequestMapping(value = "/avatar/{userId}", method = RequestMethod.GET)
	@ResponseBody
	public void downloadUserAvatarImage(@PathVariable Integer userId,
			HttpServletRequest request,
            HttpServletResponse response) {

        String appPath = Utils.getImagePath();
        User user = userService.getUser(userId);
        String fullPath = appPath + File.separator + user.getAvatar();
        File avatar = new File(fullPath);
        FileInputStream inputStream = null;
		try {
			inputStream = new FileInputStream(avatar);
			Path path;
			System.out.println("mimeType:" + Files.probeContentType(Paths.get(avatar.getPath())));
			
			 response.setHeader("Cache-Control", "no-store");
		   	response.setHeader("Pragma", "no-cache");
		    response.setDateHeader("Expires", 0);
		    response.setContentType(Files.probeContentType(Paths.get(avatar.getPath())));
		    ServletOutputStream responseOutputStream = response.getOutputStream();
		    responseOutputStream.write(IOUtils.toByteArray(inputStream));
		    responseOutputStream.flush();
		    responseOutputStream.close();
			
		    /*return ResponseEntity.ok()
		            .contentLength(avatar.length())
		            .contentType(MediaType.parseMediaType(Files.probeContentType(Paths.get(avatar.getPath()))))
		            .body(IOUtils.toByteArray(inputStream));*/
		} catch (FileNotFoundException e) {
			System.out.println("FileNotFoundException:" + e);
		} catch (IOException e) {
			System.out.println("IOException:" + e);
		}
	}
	
	
	@RequestMapping(value = "/getFriendList", method = RequestMethod.POST)
	@ResponseBody
	public List<UserRelationActual> getFriendList(HttpSession session){
		User authenticatedUser = (User) session.getAttribute("authenticatedUser");
		List<UserRelationActual> userRelationActual = new ArrayList<UserRelationActual>();
		for(UserRelationActual rel : authenticatedUser.getUserRelations()){
			if(rel.getStatus().getId() == 2){
				userRelationActual.add(rel);
			}
		}
		
		return userRelationActual;
	}
	
	@RequestMapping(value = "/friendInvite", method = RequestMethod.POST)
	@ResponseBody
	public String sendFriendInvite(@RequestParam("userId") Integer userId, HttpSession session, Model model, HttpServletRequest request){
		User authenticatedUser = (User) session.getAttribute("authenticatedUser");
		User user = userService.getUser(userId);
		if(authenticatedUser.getUserRelations() == null){
			authenticatedUser.setUserRelations(new HashSet<UserRelationActual>());
		}
		if(user != null){
			List<RequestStatus> requestStatus = requestStatusService.getAllRequestStatus();
			RequestStatus pendingRequest = null;
			for(RequestStatus req : requestStatus){
				if(req.getId() == 1){
					pendingRequest = req;
				}
			}
			UserRelationActual rel = new UserRelationActual();
			rel.setRecommendated(false);
			rel.setStatus(pendingRequest);
			rel.setToxic(false);
			rel.setUser1(authenticatedUser);
			rel.setUser2(user);
			rel.setCreationDate(new Date());
			rel = userService.addNewUserRelation(rel);
			authenticatedUser.getUserRelations().add(rel);
			if(user.getNotificationsFriends()){
				EmailSender.sendEmail(EmailSender.FRIEND_INVITE, user.getEmail(), EmailSender.friendRequest(authenticatedUser, user, request.getRequestURL().toString().replace("/user/friendInvite","")));
			}
			model.addAttribute("authenticatedUser", authenticatedUser);
			return "success";
		}
		else{
			return "failure";
		}
	}
	
	@RequestMapping(value = "/recommendFriend", method = RequestMethod.POST)
	@ResponseBody
	public String recommendFriend(@RequestParam("userId") Integer userId,@RequestParam("parameterId") Integer parameterId,
			HttpSession session, Model model){
		User authenticatedUser = (User) session.getAttribute("authenticatedUser");
		User user = userService.getUser(userId);
		if(authenticatedUser.getUserRelations() == null){
			authenticatedUser.setUserRelations(new HashSet<UserRelationActual>());
		}
		if(user != null){
			List<RecomendationParameter> parameters = recomendationParamaterService.getAllRecomendationParameters();
			UserRelationActual rel = new UserRelationActual();
			for(UserRelationActual relActual : authenticatedUser.getUserRelations()){
				if(relActual.getUser1Id() == authenticatedUser.getId() && relActual.getUser2Id() == userId){
					rel = relActual;
				}
			}
			rel.setRecommendated(true);
			for(RecomendationParameter param : parameters){
				if(param.getId() == parameterId){
					rel.setParameter(param);
				}
			}
			rel = userService.updateUserRelation(rel);
			model.addAttribute("authenticatedUser", authenticatedUser);
			return "success";
		}
		else{
			return "failure";
		}
	}
	
	@RequestMapping(value = "/acceptFriend", method = RequestMethod.POST)
	@ResponseBody
	public String acceptFriend(@RequestParam("userId") Integer userId,
			HttpSession session, Model model){
		User authenticatedUser = (User) session.getAttribute("authenticatedUser");
		User user = userService.getUser(userId);
		if(authenticatedUser.getUserRelations() == null){
			authenticatedUser.setUserRelations(new HashSet<UserRelationActual>());
		}
		if(user != null){
			UserRelationActual rel = new UserRelationActual();
			for(UserRelationActual relActual : user.getUserRelations()){
				if(relActual.getUser2Id() == authenticatedUser.getId() && relActual.getUser1Id() == userId){
					rel = relActual;
				}
			}
			List<RequestStatus> requestStatus = requestStatusService.getAllRequestStatus();
			RequestStatus acceptedRequest = null;
			for(RequestStatus req : requestStatus){
				if(req.getId() == 2){
					acceptedRequest = req;
				}
			}
			rel.setStatus(acceptedRequest);
			rel = userService.updateUserRelation(rel);
			UserRelationActual inverseRelation = new UserRelationActual();
			inverseRelation.setRecommendated(false);
			inverseRelation.setStatus(acceptedRequest);
			inverseRelation.setCreationDate(new Date());
			inverseRelation.setToxic(false);
			inverseRelation.setUser1(authenticatedUser);
			inverseRelation.setUser2(user);
			rel = userService.addNewUserRelation(inverseRelation);
			model.addAttribute("authenticatedUser", authenticatedUser);
			return "success";
		}
		else{
			return "failure";
		}
	}
	
	@RequestMapping(value = "/rejectFriend", method = RequestMethod.POST)
	@ResponseBody
	public String rejectFriend(@RequestParam("userId") Integer userId,
			HttpSession session, Model model){
		User authenticatedUser = (User) session.getAttribute("authenticatedUser");
		User user = userService.getUser(userId);
		if(authenticatedUser.getUserRelations() == null){
			authenticatedUser.setUserRelations(new HashSet<UserRelationActual>());
		}
		if(user != null){
			UserRelationActual rel = new UserRelationActual();
			for(UserRelationActual relActual : user.getUserRelations()){
				if(relActual.getUser1Id() == user.getId() && relActual.getUser2Id() == authenticatedUser.getId()){
					rel = relActual;
				}
			}
			List<RequestStatus> requestStatus = requestStatusService.getAllRequestStatus();
			RequestStatus rejectFriend = null;
			for(RequestStatus req : requestStatus){
				if(req.getId() == 3){
					rejectFriend = req;
				}
			}
			rel.setStatus(rejectFriend);
			rel = userService.updateUserRelation(rel);
			model.addAttribute("authenticatedUser", authenticatedUser);
			return "success";
		}
		else{
			return "failure";
		}
	}
	
	@RequestMapping(value = "/deleteFriend", method = RequestMethod.POST)
	@ResponseBody
	public String deleteFriend(@RequestParam("userId") Integer userId,
			HttpSession session, Model model){
		User authenticatedUser = (User) session.getAttribute("authenticatedUser");
		User user = userService.getUser(userId);
		if(authenticatedUser.getUserRelations() == null){
			authenticatedUser.setUserRelations(new HashSet<UserRelationActual>());
		}
		if(user != null){
			UserRelationActual rel = null;
			Set<UserRelationActual> newRels = new HashSet<UserRelationActual>();
			for(UserRelationActual relActual : user.getUserRelations()){
				if(relActual.getUser1Id() == user.getId() && relActual.getUser2Id() == authenticatedUser.getId()){
					rel = relActual;
				}
				else{
					newRels.add(relActual);
				}
			}
			if(rel != null){
				UserRelationHistoric historic = new UserRelationHistoric();
				historic.setCreationDate(new Date());
				historic.setParameter(rel.getParameter());
				historic.setRecommendated(rel.isRecommendated());
				historic.setStatus(rel.getStatus());
				historic.setToxic(rel.isToxic());
				historic.setUser1(rel.getUser1());
				historic.setUser2(rel.getUser2());
				if(rel.isRecommendated()){
					authenticatedUser.setRecommendationLevel(authenticatedUser.getRecommendationLevel() - 1);
					userService.updateUser(authenticatedUser);
				}
				userService.addNewUserRelationHistoric(historic);
				userService.deleteUserRelation(rel);
				user.setUserRelations(newRels);
			}
			Set<UserRelationActual> newRelsAuth = new HashSet<UserRelationActual>();
			for(UserRelationActual relActual : authenticatedUser.getUserRelations()){
				if(relActual.getUser1Id() == authenticatedUser.getId() && relActual.getUser2Id() == user.getId()){
					rel = relActual;
				}
				else{
					newRelsAuth.add(relActual);
				}
			}
			if(rel != null){
				UserRelationHistoric historic = new UserRelationHistoric();
				historic.setCreationDate(new Date());
				historic.setParameter(rel.getParameter());
				historic.setRecommendated(rel.isRecommendated());
				historic.setStatus(rel.getStatus());
				historic.setToxic(rel.isToxic());
				historic.setUser1(rel.getUser1());
				historic.setUser2(rel.getUser2());
				if(rel.isRecommendated()){
					user.setRecommendationLevel(user.getRecommendationLevel() - 1);
					userService.updateUser(user);
				}
				userService.addNewUserRelationHistoric(historic);
				userService.deleteUserRelation(rel);
				authenticatedUser.setUserRelations(newRelsAuth);
			}
			model.addAttribute("authenticatedUser", authenticatedUser);
			return "success";
		}
		else{
			return "failure";
		}
	}
	
	@RequestMapping(value = "/getPendingFriendRequests", method = RequestMethod.POST)
	@ResponseBody
	public List<UserRelationActual> getPendingFriendRequests(HttpSession session, Model model){
		User authenticatedUser = (User) session.getAttribute("authenticatedUser");
		return userService.getPendingFriendRequest(authenticatedUser.getId());
	}
	
	private void loadModelAndViewItems(ModelAndView mv){
		mv.addObject("countries", countryService.getAllCountries());
		mv.addObject("genres", genreService.getAllGenre());
		mv.addObject("platforms", platformService.getAllPlatforms());
	}
	
	
	

}
