package com.xmt.gameet.controllers;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;
import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.data.jpa.domain.Specification;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import com.xmt.gameet.entities.Game;
import com.xmt.gameet.entities.Genre;
import com.xmt.gameet.entities.Language;
import com.xmt.gameet.entities.Platform;
import com.xmt.gameet.entities.User;
import com.xmt.gameet.helpers.Day;
import com.xmt.gameet.helpers.SearchFormModel;
import com.xmt.gameet.helpers.Utils;
import com.xmt.gameet.services.GameService;
import com.xmt.gameet.services.GenreService;
import com.xmt.gameet.services.LanguageService;
import com.xmt.gameet.services.PlatformService;
import com.xmt.gameet.services.UserService;
import com.xmt.gameet.specifications.GameSpecifications;

@RequestMapping("/utils")
@RestController
public class UtilsRestController {

	@Autowired
	@Qualifier("genreService")
	GenreService genreService;
	
	@Autowired
	@Qualifier("platformService")
	PlatformService platformService;
	
	@Autowired
	@Qualifier("gameService")
	GameService gameService;
	
	@Autowired
	@Qualifier("languageService")
	LanguageService languageService;
	
	@Autowired
	@Qualifier("userService")
	UserService userService;
	
	
	@RequestMapping(value="/genres", method = RequestMethod.POST, produces = "application/json")
	public List<Genre> getGenres(){
		return genreService.getAllGenre();
	}
	
	@RequestMapping(value="/searchGenres", method = RequestMethod.POST, produces = "application/json")
	public List<Genre> searchGenres(@RequestParam(value="name", required=false) String name,
			@RequestParam(value="alreadyAdded", required=false) Boolean alreadyAdded,
			@RequestParam(value="notAddedOnly", required=false) Boolean notAddedOnly,
			@RequestParam(value="userId", required=false) Integer userId){
		return Utils.compareGenreListWithUsers(genreService.getGenresByUser(userId), genreService.searchGenres(name, alreadyAdded, notAddedOnly, userId));
	}
	
	@RequestMapping(value="/platforms", method = RequestMethod.POST, produces = "application/json")
	public List<Platform> getPlatforms(){
		return platformService.getAllPlatforms();
	}
	
	@RequestMapping(value="/searchPlatforms", method = RequestMethod.POST, produces = "application/json")
	public List<Platform> searchPlatforms(@RequestParam(value="name", required=false) String name,
			@RequestParam(value="alreadyAdded", required=false) Boolean alreadyAdded,
			@RequestParam(value="notAddedOnly", required=false) Boolean notAddedOnly,
			@RequestParam(value="userId", required=false) Integer userId){
		return Utils.comparePlatformListWithUsers(platformService.getPlatformsByUser(userId), platformService.searchPlatform(name, alreadyAdded, notAddedOnly, userId));
	}
	
	@RequestMapping(value="/searchLanguages", method = RequestMethod.POST, produces = "application/json")
	public List<Language> searchLanguages(@RequestParam(value="name", required=false) String name,
			@RequestParam(value="alreadyAdded", required=false) Boolean alreadyAdded,
			@RequestParam(value="notAddedOnly", required=false) Boolean notAddedOnly,
			@RequestParam(value="userId", required=false) Integer userId){
		return Utils.compareLanguageListWithUsers(languageService.getLanguagesByUser(userId), languageService.searchLanguages(name, alreadyAdded, notAddedOnly, userId));
	}
	
	@SuppressWarnings("unchecked")
	@RequestMapping(value="/games", method = RequestMethod.POST, produces = "application/json")
	public List<Game> getGames(@RequestParam(value="genre", required=false) Integer idGenre,
			@RequestParam(value="platform", required=false) Integer idPlatform,
			@RequestParam(value="game", required=false) String game,
			@RequestParam(value="alreadyAdded", required=false) Boolean alreadyAdded,
			@RequestParam(value="notAddedOnly", required=false) Boolean notAddedOnly,
			@RequestParam(value="userId", required=false) Integer userId){
		List<Specification<Game>> specs = new ArrayList<Specification<Game>>(); 
		if(idGenre != null && idGenre != 0){
			specs.add(GameSpecifications.addGenreFilter(idGenre));
		}
		if(idPlatform != null && idPlatform != 0) {
			specs.add(GameSpecifications.addPlatformFilter(idPlatform));
		}
		if(game != null && !game.isEmpty()){
			specs.add(GameSpecifications.addNameFilter(game));
		}
		if(alreadyAdded != null && alreadyAdded  && userId != null){
			specs.add(GameSpecifications.addAddedUserFilter(userId));
		}
		else if(notAddedOnly != null && notAddedOnly && userId != null){
			specs.add(GameSpecifications.addNotAddedUserFilter(userId));
		}
		Specification<Game>[] specifications = new Specification[specs.size()];
		int i = 0;
		for(Specification<Game> spec : specs){
			specifications[i] = spec;
			i++;
		}
		if(userId != null){
			List<Game> gamesAddedByUser = gameService.getGamesByUser(userId);
			return Utils.compareGameListWithUsers(gamesAddedByUser,gameService.searchGame(specifications));
		}
		else{
			return gameService.searchGame(specifications);
		}
	}
	
	@RequestMapping(value="/getUserProfile", method = RequestMethod.POST, produces = "application/json")
	public User getUserProfile(@RequestParam(value="userId") Integer userId,
			@RequestParam(value="userToSearch") Integer userToSearchId){
		return Utils.checkIfUsersAreFriends(userService.getUser(userId),userService.getUser(userToSearchId));
	}
	
	@RequestMapping(value="/searchUsers", method = RequestMethod.POST, produces = "application/json")
	public List<User> searchUsers(@RequestParam(value="nickname", required=false) String nickname,
			@RequestParam(value="ageFrom", required=false) Integer ageFrom,
			@RequestParam(value="ageTo", required=false) Integer ageTo,
			@RequestParam(value="languages", required=false) Integer[] languages,
			@RequestParam(value="platform", required=false) Integer platform,
			@RequestParam(value="genre", required=false) Integer genre,
			@RequestParam(value="game", required=false) Integer game,
			@RequestParam(value="days", required= false) String days,
			@RequestParam(value="hourFrom", required=false) String hourTo,
			@RequestParam(value="hourTo", required=false) String hourFrom,
			@RequestParam(value="similarUser", required=false, defaultValue="false") Boolean similarUser,
			@RequestParam(value="userId") Integer userId){
		User user = userService.getUser(userId);
		SearchFormModel searchModel = new SearchFormModel();
		if(!similarUser){
			searchModel.setNickname(nickname);
			searchModel.setAgeFrom(ageFrom);
			searchModel.setAgeTo(ageTo);
			searchModel.setPlatform(platform);
			searchModel.setGame(game);
			searchModel.setGenre(genre);
			try {
				SimpleDateFormat sdf = new SimpleDateFormat("hh:mm:ss");
				if(hourFrom != null && !hourFrom.isEmpty()){
					Date date;
					date = sdf.parse(hourFrom);
					Calendar cal = Calendar.getInstance();
					cal.setTime(date);
					Integer hFrom = cal.get(Calendar.HOUR_OF_DAY) * 60 + cal.get(Calendar.MINUTE);
					searchModel.setHourFrom(hFrom);
					searchModel.setAnyTime(false);
				}
				if(hourTo != null && !hourTo.isEmpty()){
					Date date = sdf.parse(hourTo);
					Calendar cal = Calendar.getInstance();
					cal.setTime(date);
					Integer hTo = cal.get(Calendar.HOUR_OF_DAY) * 60 + cal.get(Calendar.MINUTE);
					searchModel.setHourTo(hTo);
					searchModel.setAnyTime(false);
				}
			} catch (ParseException e) {
				System.out.println("Error al parsear las fechas." +  e);
				searchModel.setAnyTime(true);
			}
			if(languages != null){
				searchModel.setLanguages(new ArrayList<Integer>());
				for(Integer languageId : languages){
					searchModel.getLanguages().add(languageId);
				}
			}
			if(days != null && !days.isEmpty()){
				searchModel.setDays(new ArrayList<Day>());
				String[] daysSeparated = days.split("-");
				for(String dayStr : daysSeparated){
					Day day = new Day(dayStr);
					searchModel.getDays().add(day);
				}
				searchModel.setAnyDay(false);
			}
			else{
				searchModel.setAnyDay(true);
			}
		}
		else{
			searchModel.setAgeFrom(user.getAge() - 5);
			searchModel.setAgeTo(user.getAge() + 5);
			if(user.getLanguages() != null){
				List<Integer> langIds = new ArrayList<Integer>();
				for(Language lang : user.getLanguages()){
					langIds.add(lang.getId());
				}
				searchModel.setLanguages(langIds);
			}
			if(user.getGamesToPlay() != null){
				List<Integer> gameIds = new ArrayList<Integer>();
				for(Game gameobj : user.getGamesToPlay()){
					gameIds.add(gameobj.getId());
				}
				searchModel.setGames(gameIds);
			}
			if(user.getPlatformsPrefered() != null){
				List<Integer> platformsIds = new ArrayList<Integer>();
				for(Platform platformObj : user.getPlatformsPrefered()){
					platformsIds.add(platformObj.getId());
				}
				searchModel.setGames(platformsIds);
			}
			if(user.getGenresPrefered() != null){
				List<Integer> genresIds = new ArrayList<Integer>();
				for(Genre genreobj : user.getGenresPrefered()){
					genresIds.add(genreobj.getId());
				}
				searchModel.setGames(genresIds);
			}
			searchModel.setAnyDay(true);
			searchModel.setAnyTime(true);
			
		}
		searchModel.setUserSearchingId(userId);
		System.out.println("Bsqueda de usuarios:");
		return Utils.compareUserRelationsListWithUsers(user.getUserRelations(), userService.searchUsers(searchModel));
		/*if(userId != null){
			List<Game> gamesAddedByUser = gameService.getGamesByUser(userId);
			return Utils.compareGameListWithUsers(gamesAddedByUser,gameService.searchGame(specifications));
		}
		else{
			return gameService.searchGame(specifications);
		}*/
	}
	
	
}
