package com.xmt.gameet.entities;

import java.time.LocalDateTime;
import java.time.ZoneId;
import java.util.Date;
import java.util.Set;
import java.util.TimeZone;

import javax.persistence.CascadeType;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.JoinColumn;
import javax.persistence.JoinTable;
import javax.persistence.ManyToMany;
import javax.persistence.ManyToOne;
import javax.persistence.OneToMany;
import javax.persistence.Table;
import javax.persistence.Temporal;
import javax.persistence.TemporalType;
import javax.persistence.Transient;
import javax.validation.constraints.Max;
import javax.validation.constraints.Min;
import javax.validation.constraints.NotNull;
import javax.validation.constraints.Size;

import org.hibernate.validator.constraints.Email;
import org.hibernate.validator.constraints.NotEmpty;

import com.fasterxml.jackson.annotation.JsonIgnore;
import com.xmt.gameet.utils.DataBaseDataType;

@Entity
@Table(name="User")
public class User {

	@Id
	@GeneratedValue(strategy=GenerationType.AUTO)
	@Column(length=11)
	private int id;
	
	@Column(length=255)
	@NotEmpty
	@Email
	private String email;
	
	@Column(length=16)
	@NotNull
	@Size(max=45, min=5)
	private String nickname;
	
	@Column(length=45)
	@NotNull
	@Size(max=45, min=2)
	private String name;
	
	@Column(length=45)
	@NotNull
	@Size(max=45,min=2)
	private String surname;
	
	@Column(length=64)
	@Size(max=64)
	private String password;
	
	@Column(columnDefinition=DataBaseDataType.DATE, nullable=false)
	@Temporal(TemporalType.TIMESTAMP)
	private Date creationDate;
	
	@Transient
	private LocalDateTime creationDateLocal;
	
	@Column(columnDefinition=DataBaseDataType.DATE)
	@Temporal(TemporalType.TIMESTAMP)
	private Date lastLogin;
	
	@Transient
	private LocalDateTime lastLoginLocal;
	
	@Column(columnDefinition=DataBaseDataType.BOOLEAN_DEFAULT_FALSE, nullable=false)
	private boolean emailConfirmed;
	
	@Column(length=200)
	private String avatar;
	
	@Column(columnDefinition=DataBaseDataType.BOOLEAN_DEFAULT_FALSE, nullable=false)
	private boolean profileCompleted;
	
	@Column(length=3)
	@NotNull
	@Min(10) @Max(110)
	private Integer age;
	
	@Column(columnDefinition=DataBaseDataType.BOOLEAN_DEFAULT_TRUE, nullable=false)
	private boolean visible;
	
	@Column(columnDefinition=DataBaseDataType.BOOLEAN_DEFAULT_TRUE, nullable=false)
	private Boolean notificationsFriends;
	
	@Column(columnDefinition=DataBaseDataType.BOOLEAN_DEFAULT_TRUE, nullable=false)
	private Boolean notificationsEvents;
	
	@Column(columnDefinition=DataBaseDataType.BOOLEAN_DEFAULT_TRUE, nullable=false)
	private Boolean notificationsNews;
	
	@Column(columnDefinition=DataBaseDataType.BOOLEAN_DEFAULT_TRUE, nullable=false)
	private Boolean notificationsMessages;
	
	@Column(length=11)
	private Integer recommendationLevel;
	
	@Column(length=11)
	private Integer toxicityLevel;
	
	@Column(length=50)
	private String timeZoneId;
	
	@Transient
	@JsonIgnore
	private TimeZone timeZone;
	
	@Transient
	private String repeatedPassword;
	
	@Transient
	private String newPassword;
	
	@Transient
	private String confirmedPassword;
	
	@Transient
	private Boolean addedByUser;

	@ManyToOne(fetch = FetchType.EAGER, cascade =  CascadeType.PERSIST)
	@JoinColumn(name="country")
	@NotNull
	private Country country;
	
	@ManyToOne(fetch = FetchType.EAGER, cascade =  CascadeType.ALL)
	@JoinColumn(name="type")
	private UserType type;
	
	@ManyToMany(fetch=FetchType.EAGER, cascade =  CascadeType.PERSIST)
	@JoinTable(name="UserPlatformPrefered", joinColumns={@JoinColumn(name="userId")}, inverseJoinColumns={@JoinColumn(name="platformId")})
	private Set<Platform> platformsPrefered;
	
	@ManyToMany(fetch=FetchType.EAGER, cascade =  CascadeType.PERSIST)
	@JoinTable(name="UserGenrePrefered", joinColumns={@JoinColumn(name="userId")}, inverseJoinColumns={@JoinColumn(name="genreId")})
	private Set<Genre> genresPrefered;
	
	@ManyToMany(fetch=FetchType.EAGER, cascade = CascadeType.PERSIST)
	@JoinTable(name="UserGamesToPlay", joinColumns={@JoinColumn(name="userId")}, inverseJoinColumns={@JoinColumn(name="gameId")})
	private Set<Game> gamesToPlay;
	
	@ManyToMany(fetch=FetchType.EAGER)
	@JoinTable(name="UserAchievements", joinColumns={@JoinColumn(name="userId")}, inverseJoinColumns={@JoinColumn(name="achievementId")})
	private Set<Achievement> achievements;
	
	@ManyToMany(fetch=FetchType.EAGER, cascade =  CascadeType.PERSIST)
	@JoinTable(name="UserLanguage", joinColumns={@JoinColumn(name="userId")}, inverseJoinColumns={@JoinColumn(name="languageId")})
	private Set<Language> languages;
	
	@OneToMany(fetch = FetchType.EAGER, mappedBy= "user", cascade =  CascadeType.PERSIST)
	private Set<Timetable> timetable;
	
	@OneToMany(fetch = FetchType.EAGER, mappedBy= "user1", cascade =  CascadeType.PERSIST)
	private Set<MessageMail> messagesSent;
	
	@OneToMany(fetch = FetchType.EAGER, mappedBy= "user2", cascade =  CascadeType.PERSIST)
	private Set<MessageMail> messagesReceived;
	
	@OneToMany(fetch = FetchType.EAGER, mappedBy= "user1", cascade =  CascadeType.PERSIST)
	private Set<UserRelationActual> userRelations;
	
	@OneToMany(fetch = FetchType.LAZY, mappedBy= "user1", cascade =  CascadeType.PERSIST)
	private Set<UserRelationHistoric> userRelationHistoric;
	
	@OneToMany(fetch = FetchType.EAGER, mappedBy= "user1", cascade =  CascadeType.PERSIST)
	private Set<Notification> userNotifications;
	
	public int getId() {
		return id;
	}

	public void setId(int id) {
		this.id = id;
	}

	public String getEmail() {
		return email;
	}

	public void setEmail(String email) {
		this.email = email;
	}

	public String getNickname() {
		return nickname;
	}

	public void setNickname(String nickname) {
		this.nickname = nickname;
	}

	public String getName() {
		return name;
	}

	public void setName(String name) {
		this.name = name;
	}

	public String getSurname() {
		return surname;
	}

	public void setSurname(String surname) {
		this.surname = surname;
	}

	public String getPassword() {
		return password;
	}

	public void setPassword(String password) {
		this.password = password;
	}

	public Date getCreationDate() {
		return creationDate;
	}

	public void setCreationDate(Date creationDate) {
		this.creationDate = creationDate;
	}

	public Date getLastLogin() {
		return lastLogin;
	}

	public void setLastLogin(Date lastLogin) {
		this.lastLogin = lastLogin;
	}

	public boolean isEmailConfirmed() {
		return emailConfirmed;
	}

	public void setEmailConfirmed(boolean emailConfirmed) {
		this.emailConfirmed = emailConfirmed;
	}

	public String getAvatar() {
		return avatar;
	}

	public void setAvatar(String avatar) {
		this.avatar = avatar;
	}

	public boolean isProfileCompleted() {
		return profileCompleted;
	}

	public void setProfileCompleted(boolean profileCompleted) {
		this.profileCompleted = profileCompleted;
	}

	public Integer getAge() {
		return age;
	}

	public void setAge(Integer age) {
		this.age = age;
	}

	public boolean isVisible() {
		return visible;
	}

	public void setVisible(boolean visible) {
		this.visible = visible;
	}

	public UserType getType() {
		return type;
	}

	public void setType(UserType type) {
		this.type = type;
	}
	
	public String getTimeZoneId() {
		return timeZoneId;
	}

	public void setTimeZoneId(String timeZoneId) {
		this.timeZoneId = timeZoneId;
	}
	
	public TimeZone getTimeZone() {
		if(timeZoneId != null){
			return TimeZone.getTimeZone(this.timeZoneId);
		}
		else{
			return null;
		}
	}

	public void setTimeZone(TimeZone timeZone) {
		this.timeZone = timeZone;
	}

	public Set<Platform> getPlatformsPrefered() {
		return platformsPrefered;
	}

	public void setPlatformsPrefered(Set<Platform> platformsPrefered) {
		this.platformsPrefered = platformsPrefered;
	}

	public Set<Genre> getGenresPrefered() {
		return genresPrefered;
	}

	public void setGenresPrefered(Set<Genre> genresPrefered) {
		this.genresPrefered = genresPrefered;
	}

	public Set<Game> getGamesToPlay() {
		return gamesToPlay;
	}

	public void setGamesToPlay(Set<Game> gamesToPlay) {
		this.gamesToPlay = gamesToPlay;
	}

	public Set<Achievement> getAchievements() {
		return achievements;
	}

	public void setAchievements(Set<Achievement> achievements) {
		this.achievements = achievements;
	}

	public Set<Language> getLanguages() {
		return languages;
	}

	public void setLanguages(Set<Language> languages) {
		this.languages = languages;
	}

	public Set<Timetable> getTimetable() {
		return timetable;
	}

	public void setTimetable(Set<Timetable> timetable) {
		this.timetable = timetable;
	}

	public Set<UserRelationActual> getUserRelations() {
		return userRelations;
	}

	public void setUserRelations(Set<UserRelationActual> userRelations) {
		this.userRelations = userRelations;
	}

	@JsonIgnore
	public Set<UserRelationHistoric> getUserRelationHistoric() {
		return userRelationHistoric;
	}

	public void setUserRelationHistoric(Set<UserRelationHistoric> userRelationHistoric) {
		this.userRelationHistoric = userRelationHistoric;
	}

	public Set<Notification> getUserNotifications() {
		return userNotifications;
	}

	public void setUserNotifications(Set<Notification> userNotifications) {
		this.userNotifications = userNotifications;
	}

	public LocalDateTime getCreationDateLocal() {
		LocalDateTime ldt = LocalDateTime.ofInstant(this.creationDate.toInstant(), ZoneId.systemDefault());
		return ldt;
	}

	public void setCreationDateLocal(LocalDateTime creationDateLocal) {
		this.creationDateLocal = creationDateLocal;
	}

	public LocalDateTime getLastLoginLocal() {
		if(this.lastLogin != null){
			LocalDateTime ldt = LocalDateTime.ofInstant(this.lastLogin.toInstant(), ZoneId.systemDefault());
			return ldt;
		}
		return null;
	}

	public void setLastLoginLocal(LocalDateTime lastLoginLocal) {
		this.lastLoginLocal = lastLoginLocal;
	}

	public Integer getRecommendationLevel() {
		return recommendationLevel;
	}

	public void setRecommendationLevel(Integer recommendationLevel) {
		this.recommendationLevel = recommendationLevel;
	}

	public Integer getToxicityLevel() {
		return toxicityLevel;
	}

	public void setToxicityLevel(Integer toxicityLevel) {
		this.toxicityLevel = toxicityLevel;
	}

	public Country getCountry() {
		return country;
	}

	public void setCountry(Country country) {
		this.country = country;
	}

	public String getRepeatedPassword() {
		return repeatedPassword;
	}

	public void setRepeatedPassword(String repeatedPassword) {
		this.repeatedPassword = repeatedPassword;
	}

	public String getNewPassword() {
		return newPassword;
	}

	public void setNewPassword(String newPassword) {
		this.newPassword = newPassword;
	}

	public String getConfirmedPassword() {
		return confirmedPassword;
	}

	public void setConfirmedPassword(String confirmedPassword) {
		this.confirmedPassword = confirmedPassword;
	}

	public boolean getNotificationsFriends() {
		return notificationsFriends;
	}

	public void setNotificationsFriends(boolean notificationsFriends) {
		this.notificationsFriends = notificationsFriends;
	}

	public boolean getNotificationsEvents() {
		return notificationsEvents;
	}

	public void setNotificationsEvents(boolean notificationsEvents) {
		this.notificationsEvents = notificationsEvents;
	}

	public boolean getNotificationsNews() {
		return notificationsNews;
	}

	public void setNotificationsNews(boolean notificationsNews) {
		this.notificationsNews = notificationsNews;
	}

	public boolean getNotificationsMessages() {
		return notificationsMessages;
	}

	public void setNotificationsMessages(boolean notificationsMessages) {
		this.notificationsMessages = notificationsMessages;
	}
	
	public void checkIfProfileCompleted(){
		System.out.println("PERFIL COMPLETO?");
		boolean completed = true;
		if(this.email == null){
			System.out.println("1");
			completed = false;
		}
		if(!this.emailConfirmed){
			System.out.println("2");
			completed = false;
		}
		if(this.name == null){
			System.out.println("3");
			completed = false;
		}
		if(this.surname == null){
			System.out.println("4");
			completed = false;
		}
		if(this.age == null){
			System.out.println("5");
			completed = false;
		}
		if(this.country == null){
			System.out.println("6");
			completed = false;
		}
		if(this.nickname == null){
			System.out.println("7");
			completed = false;
		}
		if(this.getLanguages() == null || this.getLanguages().size() < 1){
			System.out.println("LENGUAJES VACIO");
			completed = false;
		}
		if(this.getGenresPrefered() == null || this.getGenresPrefered().size() < 1){
			System.out.println("GENEROS VACIO");
			completed = false;
		}
		if(this.getPlatformsPrefered() == null || this.getPlatformsPrefered().size() < 1){
			System.out.println("PLATAFORMAS VACIO");
			completed = false;
		}
		this.profileCompleted = completed;
	}

	public Boolean getAddedByUser() {
		return addedByUser;
	}

	public void setAddedByUser(Boolean addedByUser) {
		this.addedByUser = addedByUser;
	}

	public Set<MessageMail> getMessagesSent() {
		return messagesSent;
	}

	public void setMessagesSent(Set<MessageMail> messagesSent) {
		this.messagesSent = messagesSent;
	}

	public Set<MessageMail> getMessagesReceived() {
		return messagesReceived;
	}

	public void setMessagesReceived(Set<MessageMail> messagesReceived) {
		this.messagesReceived = messagesReceived;
	}
	
	
	
	
	/*@Override
	public boolean equals(Object obj){
		if(obj == null) return false;
		if(this == obj) return true;
		if(!(obj instanceof User)) return false;
		
		final User user = (User) obj;
		return new EqualsBuilder().
				append(user.getId(), this.getId()).
				append(user.getEmail(), this.getEmail()).
				append(user.getAge(), this.getAge()).
				append(user.getAvatar(), this.getAvatar()).
				append(user.getCountry(), this.getCountry()).
				append(user.getCreationDate(), this.getCreationDate()).
				append(user.getGamesToPlay(), this.getGamesToPlay()).
				//append(user.getGenresPrefered(), this.getGenresPrefered()).
				//append(user.getLanguages(), this.getLanguages()).
				append(user.getName(), this.getName()).
				append(user.getNickname(), this.getNickname()).
				append(user.getPassword(), this.getPassword()).
				//append(user.getPlatformsPrefered(), this.getPlatformsPrefered()).
				append(user.getSurname(), this.getSurname()).
				//append(user.getTimetable(), this.getTimetable()).
				append(user.getType(), this.getType()).
				//append(user.getUserNotifications(), this.getUserNotifications()).
				//append(user.getUserRelationHistoric(), this.getUserRelationHistoric()).
				//append(user.getUserRelations(), this.getUserRelations()).
				//append(user.getTimeZoneId(), this.getTimeZoneId()).
				isEquals();
	}
	
	@Override
	public int hashCode(){
		return new HashCodeBuilder(17,31).
				append(this.getId()).
				append(this.getEmail()).
				append(this.getAge()).
				append(this.getAvatar()).
				append(this.getCountry()).
				append(this.getCreationDate()).
				append(this.getGamesToPlay()).
				//append(this.getGenresPrefered()).
				//append(this.getLanguages()).
				append(this.getName()).
				append(this.getNickname()).
				append(this.getPassword()).
				//append(this.getPlatformsPrefered()).
				append(this.getSurname()).
				//append(this.getTimetable()).
				append(this.getType()).
				//append(this.getUserNotifications()).
				//append(this.getUserRelationHistoric()).
				//append(this.getUserRelations()).
				//append(this.getTimeZoneId()).
				toHashCode();
	}
	*/
	
	
	
	
	
	
	
	
	

	
	
	
}
