package com.xmt.gameet.helpers;

import java.io.File;
import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;
import java.util.Collection;
import java.util.List;

import org.springframework.security.crypto.bcrypt.BCryptPasswordEncoder;

import com.xmt.gameet.entities.Game;
import com.xmt.gameet.entities.Genre;
import com.xmt.gameet.entities.Language;
import com.xmt.gameet.entities.Platform;
import com.xmt.gameet.entities.Timetable;
import com.xmt.gameet.entities.User;
import com.xmt.gameet.entities.UserRelation;
import com.xmt.gameet.entities.UserRelationActual;
import com.xmt.gameet.entities.UserType;

public class Utils {

	public static List<Game> compareGameListWithUsers(List<Game> gamesAdded, List<Game> gamesList){
		for(Game game : gamesList){
			boolean found = false;
			for(Game gameAdded : gamesAdded){
				if(gameAdded.getId() == game.getId()){
					found = true;
				}
			}
			if(found){
				game.setAddedByUser(true);
			}
		}
		
		return gamesList;
	}
	
	public static List<Genre> compareGenreListWithUsers(List<Genre> genresAdded, List<Genre> genresList){
		for(Genre genre : genresList){
			boolean found = false;
			for(Genre genreAdded : genresAdded){
				if(genreAdded.getId() == genre.getId()){
					found = true;
				}
			}
			if(found){
				genre.setAddedByUser(true);
			}
		}
		
		return genresList;
	}
	
	public static List<Platform> comparePlatformListWithUsers(List<Platform> platformsAdded, List<Platform> platformsList){
		for(Platform platform : platformsList){
			boolean found = false;
			for(Platform platformAdded : platformsAdded){
				if(platformAdded.getId() == platform.getId()){
					found = true;
				}
			}
			if(found){
				platform.setAddedByUser(true);
			}
		}
		
		return platformsList;
	}
	
	public static List<Language> compareLanguageListWithUsers(List<Language> languagesAdded, List<Language> languagesList){
		for(Language language : languagesList){
			boolean found = false;
			for(Language languageAdded : languagesAdded){
				if(languageAdded.getId() == language.getId()){
					found = true;
				}
			}
			if(found){
				language.setAddedByUser(true);
			}
		}
		
		return languagesList;
	}
	
	public static List<User> compareUserRelationsListWithUsers(Collection<? extends UserRelation> usersAdded, List<User> usersList){
		for(User user : usersList){
			boolean found = false;
			for(UserRelation userAdded : usersAdded){
				if(userAdded.getUser2Id() != null){
					if(userAdded.getUser2Id() == user.getId()){
						found = true;
					}
				}
			}
			if(found){
				user.setAddedByUser(true);
			}
		}
		
		return usersList;
	}
	
	public static User checkIfUsersAreFriends(User user1, User user2){
		boolean added = false;
		for(UserRelationActual rel : user1.getUserRelations()){
			if(user2.getId() == rel.getUser2Id()){
				added = true;
			}
		}
		user2.setAddedByUser(added);
		return user2;
	}
	
	public static String getImagePath(){
		String rootPath = System.getProperty("catalina.home");
		String avatarImagesPath = rootPath + File.separator + "gameet" + File.separator + "users" + File.separator + "avatars";
		return avatarImagesPath;
	}
	
	public static UserType getNormalUserType(){
		UserType type = new UserType();
		type.setId(1);
		type.setName("USER");
		type.setDescriptionEs("Usuario");
		type.setDescriptionEN("User");
		return type;
	}
	
	public static Timetable createTimetable(String hour, String day){
		Timetable timetable = new Timetable();
		timetable.setDay(day);
		String initialHour = hour.split("-")[0];
		Integer minutes = Integer.parseInt(initialHour.split(":")[0]) * 60 + Integer.parseInt(initialHour.split(":")[1]);
		timetable.setInitialTime(minutes);
		String finalHour = hour.split("-")[1];
		minutes = Integer.parseInt(finalHour.split(":")[0]) * 60 + Integer.parseInt(finalHour.split(":")[1]);
		timetable.setEndingTime(minutes);
		
		return timetable;
	}
	
	 public static String getEncryptedPassword(String password)   {  
		BCryptPasswordEncoder passwordEncoder = new BCryptPasswordEncoder();
		return passwordEncoder.encode(password);	 
	 }
	
}
