package com.xmt.gameet.services.impl;

import java.util.ArrayList;
import java.util.List;

import javax.annotation.Resource;

import org.springframework.cache.annotation.Cacheable;
import org.springframework.data.jpa.domain.Specification;
import org.springframework.data.jpa.domain.Specifications;
import org.springframework.stereotype.Service;

import com.xmt.gameet.entities.Genre;
import com.xmt.gameet.repositories.GenreRepository;
import com.xmt.gameet.services.GenreService;
import com.xmt.gameet.specifications.GenreSpecifications;
import com.xmt.gameet.specifications.PlatformSpecifications;

@Service("genreService")
public class GenreServiceImpl implements GenreService {

	@Resource
	private GenreRepository genreRepository;
	
	public Genre create(Genre genre) {
		return genreRepository.saveAndFlush(genre);
	}

	public void delete(Genre genre) {
		genreRepository.delete(genre);
	}
	@Cacheable(value="genresFindCache")
	public List<Genre> getAllGenre() {
		return new ArrayList<Genre>(genreRepository.findAll(GenreSpecifications.orderByName()));
	}

	public Genre update(Genre genre) {
		return genreRepository.saveAndFlush(genre);
	}

	public Genre getGenre(Integer id) {
		return genreRepository.findOne(id);
	}

	public List<Genre> searchGenres(String name, Boolean alreadyAdded, Boolean notAdded, Integer userId) {
		List<Genre> genres = null;
		Specifications<Genre> spec = null;
		List<Specification<Genre>> specList = new ArrayList<Specification<Genre>>();
		specList.add(GenreSpecifications.orderByName());
		if(name != null && !name.isEmpty()){
			specList.add(GenreSpecifications.addNameFilter(name));
		}
		if(alreadyAdded != null && alreadyAdded && userId != null){
			specList.add(GenreSpecifications.addAddedUserFilter(userId));
		}
		if(notAdded != null && notAdded  && userId != null){
			specList.add(GenreSpecifications.addNotAddedUserFilter(userId));
		}
		if(!specList.isEmpty()){
			spec = Specifications.where(specList.get(0));
			if(specList.size() > 1){
				for(int i = 1; i < specList.size() ; i++){
					spec.and(specList.get(i));
				}
			}
			genres = genreRepository.findAll(spec);
		}
		else{
			genres = genreRepository.findAll();
		}
		return genres;
	}

	public List<Genre> getGenresByUser(Integer userId) {
		return genreRepository.findAll(GenreSpecifications.addAddedUserFilter(userId));
	}

}
