package com.xmt.gameet.services.impl;

import java.util.ArrayList;
import java.util.List;

import javax.annotation.Resource;

import org.springframework.data.jpa.domain.Specification;
import org.springframework.data.jpa.domain.Specifications;
import org.springframework.stereotype.Service;

import com.xmt.gameet.entities.Language;
import com.xmt.gameet.repositories.LanguageRepository;
import com.xmt.gameet.services.LanguageService;
import com.xmt.gameet.specifications.GenreSpecifications;
import com.xmt.gameet.specifications.LanguageSpecifications;
import com.xmt.gameet.specifications.PlatformSpecifications;

@Service("languageService")
public class LanguageServiceImpl implements LanguageService{

	@Resource
	private LanguageRepository languageRepository;
	
	public Language create(Language language) {
		return languageRepository.saveAndFlush(language);
	}

	public void delete(Language language) {
		languageRepository.delete(language);
	}

	public List<Language> getAllLanguage() {
		return languageRepository.findAll(LanguageSpecifications.orderByName());
	}

	public Language update(Language language) {
		return languageRepository.saveAndFlush(language);
	}

	public Language getLanguage(Integer id) {
		return languageRepository.findOne(id);
	}

	public List<Language> searchLanguages(String name, Boolean alreadyAdded, Boolean notAdded, Integer userId) {
		List<Language> languages = null;
		Specifications<Language> spec = null;
		List<Specification<Language>> specList = new ArrayList<Specification<Language>>();
		specList.add(LanguageSpecifications.orderByName());
		if(name != null && !name.isEmpty()){
			specList.add(LanguageSpecifications.addNameFilter(name));
		}
		if(alreadyAdded != null && alreadyAdded && userId != null){
			specList.add(LanguageSpecifications.addAddedUserFilter(userId));
		}
		if(notAdded != null && notAdded  && userId != null){
			specList.add(LanguageSpecifications.addNotAddedUserFilter(userId));
		}
		if(!specList.isEmpty()){
			spec = Specifications.where(specList.get(0));
			if(specList.size() > 1){
				for(int i = 1; i < specList.size() ; i++){
					spec.and(specList.get(i));
				}
			}
			languages = languageRepository.findAll(spec);
		}
		else{
			languages = languageRepository.findAll();
		}
		return languages;
	}

	public List<Language> getLanguagesByUser(Integer userId) {
		return languageRepository.findAll(LanguageSpecifications.addAddedUserFilter(userId));
	}

}
