package com.xmt.gameet.services.impl;

import java.util.ArrayList;
import java.util.List;

import javax.annotation.Resource;

import org.springframework.cache.annotation.Cacheable;
import org.springframework.data.jpa.domain.Specification;
import org.springframework.data.jpa.domain.Specifications;
import org.springframework.stereotype.Service;

import com.xmt.gameet.entities.Platform;
import com.xmt.gameet.repositories.PlatformRepository;
import com.xmt.gameet.services.PlatformService;
import com.xmt.gameet.specifications.PlatformSpecifications;

@Service("platformService")
public class PlatformServiceImpl implements PlatformService{

	@Resource
	private PlatformRepository platformRepository;
	
	public Platform create(Platform platform) {
		return platformRepository.save(platform);
	}

	public void delete(Platform platform) {
		platformRepository.delete(platform);
	}
	@Cacheable(value="platformsFindCache")
	public List<Platform> getAllPlatforms() {
		return new ArrayList<Platform>(platformRepository.findAll(PlatformSpecifications.orderByName()));
	}

	public Platform update(Platform platform) {
		return platformRepository.save(platform);
	}

	public Platform getPlatform(Integer id) {
		return platformRepository.findOne(id);
	}


	public List<Platform> searchPlatform(String name, Boolean alreadyAdded, Boolean notAdded, Integer userId) {
		List<Platform> platforms = null;
		Specifications<Platform> spec = null;
		List<Specification<Platform>> specList = new ArrayList<Specification<Platform>>();
		specList.add(PlatformSpecifications.orderByName());
		if(name != null && !name.isEmpty()){
			specList.add(PlatformSpecifications.addNameFilter(name));
		}
		if(alreadyAdded != null && alreadyAdded && userId != null){
			specList.add(PlatformSpecifications.addAddedUserFilter(userId));
		}
		if(notAdded != null && notAdded  && userId != null){
			specList.add(PlatformSpecifications.addNotAddedUserFilter(userId));
		}
		if(!specList.isEmpty()){
			spec = Specifications.where(specList.get(0));
			if(specList.size() > 1){
				for(int i = 1; i < specList.size() ; i++){
					spec.and(specList.get(i));
				}
			}
			platforms = platformRepository.findAll(spec);
		}
		else{
			platforms = platformRepository.findAll();
		}
		return platforms;
	}

	public List<Platform> getPlatformsByUser(Integer userId) {
		return platformRepository.findAll(PlatformSpecifications.addAddedUserFilter(userId));
	}

}
