package com.xmt.gameet.services.impl;

import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import javax.annotation.Resource;

import org.apache.commons.validator.routines.EmailValidator;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import com.xmt.gameet.entities.Notification;
import com.xmt.gameet.entities.User;
import com.xmt.gameet.entities.UserRelation;
import com.xmt.gameet.entities.UserRelationActual;
import com.xmt.gameet.entities.UserRelationHistoric;
import com.xmt.gameet.entities.UserType;
import com.xmt.gameet.helpers.EmailSender;
import com.xmt.gameet.helpers.ErrorMessages;
import com.xmt.gameet.helpers.SearchFormModel;
import com.xmt.gameet.helpers.Utils;
import com.xmt.gameet.repositories.UserRelationActualRepository;
import com.xmt.gameet.repositories.UserRelationHistoricRepository;
import com.xmt.gameet.repositories.UserRepository;
import com.xmt.gameet.repositories.UserStatusRepository;
import com.xmt.gameet.repositories.UserTypeRepository;
import com.xmt.gameet.services.UserService;
import com.xmt.gameet.specifications.UserRelationActualSpecifications;
import com.xmt.gameet.specifications.UserSpecifications;

@Service("userService")
@Transactional
public class UserServiceImpl implements UserService {

	@Resource
	private UserRepository userRepository;
	
	@Resource
	private UserRelationActualRepository userRelationActualRepository;
	
	@Resource
	private UserRelationHistoricRepository userRelationHistoricRepository;
	
	@Resource
	private UserStatusRepository userStatusRepository;
	
	@Resource
	private UserTypeRepository userTypeRepository;
	
	
	public User createUser(User user) {
		if(user.getCreationDate() == null){
			user.setCreationDate(new Date());
		}
		
		return userRepository.saveAndFlush(user);
	}

	public void deleteUser(User user) {
		userRepository.delete(user);
	}

	public List<User> getAllUsers() {
		return userRepository.findAll();
	}

	public User updateUser(User user) {
		user.checkIfProfileCompleted();
		user = userRepository.saveAndFlush(user);
		userRepository.flush();
		return user;
	}

	public User getUser(Integer id) {
		User user = userRepository.findOne(id);
		if(user != null){
			if(user.getUserRelations() != null){
				for(UserRelationActual rel : user.getUserRelations()){
					rel.getUser1().getId();
					rel.getUser2().getId();
				}
			}
			if(user.getUserNotifications() != null){
				for(Notification notif : user.getUserNotifications()){
					notif.getUser1().getId();
					notif.getUser2().getId();
					notif.getGame().getId();
				}
			}
		}
		return user;
	}

	public User findUserByCredentials(String email, String password) {
		// TODO Auto-generated method stub
		return null;
	}

	public List<User> getFriendList(Integer user) {
		return userRepository.findAll(UserSpecifications.addUserRelation(user));
	}

	public UserRelationActual addNewUserRelation(UserRelationActual userRelation) {
		return userRelationActualRepository.saveAndFlush(userRelation);
	}

	public void deleteUserRelation(UserRelationActual userRelation) {
		userRelationActualRepository.delete(userRelation);
	}

	public List<UserRelation> getCurrentUserRelationsByUser(User user) {
		// TODO Auto-generated method stub
		return null;
	}

	public List<UserRelation> getHistoricUserRelationsByUser(User user) {
		// TODO Auto-generated method stub
		return null;
	}

	public UserRelationActual updateUserRelation(UserRelationActual userRelation) {
		return userRelationActualRepository.saveAndFlush(userRelation);
	}

	public User findUserByEmail(String email) {
		User user = userRepository.findOne(UserSpecifications.addEmailFilter(email));
		if(user != null){
			if(user.getUserRelations() != null){
				for(UserRelationActual rel : user.getUserRelations()){
					rel.getUser1().getId();
					rel.getUser2().getId();
				}
			}
			if(user.getUserNotifications() != null){
				for(Notification notif : user.getUserNotifications()){
					notif.getUser1().getId();
					notif.getUser2().getId();
					notif.getGame().getId();
				}
			}
		}
		return user;
	}

	public Map<String, String> verifyFirstRegistration(User user) {
		Map<String,String> errors = new HashMap<String,String>();
		if(user.getPassword() == null || user.getPassword().isEmpty()){
			errors.put("passwordError", ErrorMessages.PASSWORD_EMPTY);
		}
		else{
			if(!verifyPasswordPattern(user.getPassword())){
				errors.put("passwordError", ErrorMessages.PASSWORD_PATTERN_INCORRECT);
			}
			else{
				if(!user.getPassword().equals(user.getRepeatedPassword())){
					errors.put("passwordError", ErrorMessages.PASSWORD_MISMATCH);
				}
			}
		}
		if(user.getEmail() == null || user.getEmail().isEmpty()){
			errors.put("emailError", ErrorMessages.EMAIL_EMPTY);
		}
		else{
			if(!verifyEmailPattern(user.getEmail())){
				errors.put("emailError", ErrorMessages.EMAIL_PATTERN_INCORRECT);
			}
			else{
				if(userRepository.findByEmail(user.getEmail()) != null){
					errors.put("emailError", ErrorMessages.EMAIL_ALREADY_IN_USE);
				}
			}
		}
		if(user.getNickname() == null || user.getNickname().isEmpty()){
			errors.put("nicknameError", ErrorMessages.USERNAME_EMPTY);
		}
		return errors;
	}
	
	public Map<String, String> verifyPasswords(User user){
		Map<String,String> errors = new HashMap<String,String>();
		if(user.getPassword().equals(user.getConfirmedPassword())){
			errors.put("passwordError", ErrorMessages.PASSWORD_INCORRECT);
		}
		else{
			if(!verifyPasswordPattern(user.getNewPassword())){
				errors.put("newPasswordError", ErrorMessages.PASSWORD_PATTERN_INCORRECT);
			}
			else{
				if(!user.getNewPassword().equals(user.getRepeatedPassword())){
					errors.put("newPasswordError", ErrorMessages.PASSWORD_MISMATCH);
				}
			}
		}
		
		return errors;
	}
	
	private boolean verifyPasswordPattern(String password){
		String regExCaps = "[A-Z]+";
		Pattern patternCaps = Pattern.compile(regExCaps);
		Matcher matcherCaps = patternCaps.matcher(password);
		int countCaps = 0;
		while (matcherCaps.find()) {
		   countCaps+=matcherCaps.group(0).length();
		}
		String regExNums = "[0-9]+";
		Pattern patternNums = Pattern.compile(regExNums);
		Matcher matcherNums = patternNums.matcher(password);
		int countNums = 0;
		while (matcherNums.find()) {
		   countNums+=matcherNums.group(0).length();
		}
		return (countCaps > 0 && countNums > 0);
	}

	private boolean verifyEmailPattern(String email){
		return EmailValidator.getInstance().isValid(email);
	}

	public User registerUser(User user, String localAddr) {
		if(user.getCreationDate() == null){
			user.setCreationDate(new Date());
		}
		user.setPassword(Utils.getEncryptedPassword(user.getPassword()));
		user.setVisible(true);
		user.setNotificationsEvents(true);
		user.setNotificationsFriends(true);
		user.setNotificationsMessages(true);
		user.setNotificationsNews(true);
		user = userRepository.saveAndFlush(user);
		user.setType(userTypeRepository.findOne(1));
		userRepository.saveAndFlush(user);
		System.out.println("CORREO: " + user.getEmail());
		EmailSender.sendEmail(EmailSender.CONFIRMATION_SUBJ, user.getEmail(), EmailSender.emailConfirmationBody(user, localAddr));
		return user;
	}

	public boolean confirmUserEmail(User user) {
		return userRepository.confirmMail(user.getId()) > 0;
	}

	public UserType getUserType(Integer id) {
		return userTypeRepository.findOne(id);
	}

	public List<User> searchUsers(SearchFormModel searchModel) {
		List<User> users = userRepository.findAll(UserSpecifications.addSearchFilters(searchModel));
		if(users != null){
			for(User user : users){
				if(user.getUserRelations() != null){
					for(UserRelationActual rel : user.getUserRelations()){
						rel.getUser1Id();
						rel.getUser2Id();
					}
				}
				if(user.getUserNotifications() != null){
					for(Notification notif : user.getUserNotifications()){
						notif.getUser1().getId();
						notif.getUser2().getId();
						notif.getGame().getId();
					}
				}
			}
		}
		return users;
	}

	public User getUserByNickname(String nickname) {
		return userRepository.findOne(UserSpecifications.addNicknameFilter(nickname));
	}

	public UserRelationHistoric addNewUserRelationHistoric(UserRelationHistoric userRelationHistoric) {
		return userRelationHistoricRepository.saveAndFlush(userRelationHistoric);
	}

	public List<UserRelationActual> getPendingFriendRequest(Integer userId) {
		List<UserRelationActual>  rels = userRelationActualRepository.findAll(UserRelationActualSpecifications.getPendingRelations(userId));
		for(UserRelationActual rel : rels){
			rel.getUser1().getId();
			rel.getUser2().getId();
			System.out.println("USUARIO1: " +rel.getUser1().getId() );
			System.out.println("USUARIO2: " +rel.getUser2().getId() );
		}
		return rels;
	}
	
	
}
