package com.xmt.gameet.specifications;

import javax.persistence.criteria.CriteriaBuilder;
import javax.persistence.criteria.CriteriaQuery;
import javax.persistence.criteria.Expression;
import javax.persistence.criteria.Join;
import javax.persistence.criteria.JoinType;
import javax.persistence.criteria.Predicate;
import javax.persistence.criteria.Root;
import javax.persistence.criteria.Subquery;

import org.springframework.data.jpa.domain.Specification;

import com.xmt.gameet.entities.Game;
import com.xmt.gameet.entities.Game_;
import com.xmt.gameet.entities.Genre;
import com.xmt.gameet.entities.Platform;
import com.xmt.gameet.entities.User;
import com.xmt.gameet.entities.User_;

public class GameSpecifications {

	public static Specification<Game> orderByName(){
		return new Specification<Game>(){

			public Predicate toPredicate(Root<Game> root, CriteriaQuery<?> query, CriteriaBuilder cb) {
				query.orderBy(cb.asc(root.get(Game_.name)));
				return cb.conjunction();
			}
			
		};
	}
	
	public static Specification<Game> addGenreFilter(final Integer genreId){
		return new Specification<Game>(){

			public Predicate toPredicate(Root<Game> root, CriteriaQuery<?> query, CriteriaBuilder cb) {
				query.distinct(true);
				 Predicate predicate = cb.conjunction();
	             if (genreId != null) {
	                  Join<Game, Genre> join = root.join("genres");
	                  predicate.getExpressions().add(join.get("id").in(genreId) );
	             }
	             return predicate;
			}
			
		};
	}
	
	public static Specification<Game> addPlatformFilter(final Integer platformId){
		return new Specification<Game>(){

			public Predicate toPredicate(Root<Game> root, CriteriaQuery<?> query, CriteriaBuilder cb) {
				query.distinct(true);
				 Predicate predicate = cb.conjunction();
	             if (platformId != null) {
	                  Join<Game, Platform> join = root.join("platforms");
	                  predicate.getExpressions().add(join.get("id").in(platformId) );
	             }
	             return predicate;
			}
			
		};
	}
	
	public static Specification<Game> addNameFilter(final String name){
		return new Specification<Game>(){

			public Predicate toPredicate(Root<Game> root, CriteriaQuery<?> query, CriteriaBuilder cb) {
				Expression<String> path = root.get("name");
				return cb.like(path, "%" + name + "%");
			}
			
		};
	}
	
	public static Specification<Game> addAddedUserFilter(final Integer user){
		return new Specification<Game>(){

			public Predicate toPredicate(Root<Game> root, CriteriaQuery<?> query, CriteriaBuilder cb) {
				query.distinct(true);
				Predicate predicate = cb.conjunction();
	            Join<Game, User> join = root.join(Game_.users, JoinType.INNER);
	            predicate.getExpressions().add(join.get(User_.id).in(user));
	            return predicate;
			}
			
		};
	}
	
	public static Specification<Game> addNotAddedUserFilter(final Integer user){
		return new Specification<Game>(){

			public Predicate toPredicate(Root<Game> root, CriteriaQuery<?> query, CriteriaBuilder cb) {
				query.distinct(true);
				
				Subquery<Integer> sq = query.subquery(Integer.class);
				Root<Game> rootSQ = sq.from(Game.class); 
				Join<Game, User> joinSQ = rootSQ.join(Game_.users, JoinType.INNER);
				sq.where(joinSQ.get(User_.id).in(user));
				sq.select(rootSQ.get(Game_.id));
				sq.distinct(true);
				
				Predicate predicate = cb.conjunction();
				Join<Game, User> join = root.join(Game_.users, JoinType.LEFT);
	            predicate.getExpressions().add(cb.not(root.get(Game_.id).in(sq)));
	            return predicate;
			}
			
		};
	}
	
}
