package com.xmt.gameet.specifications;

import javax.persistence.criteria.CriteriaBuilder;
import javax.persistence.criteria.CriteriaQuery;
import javax.persistence.criteria.Expression;
import javax.persistence.criteria.Join;
import javax.persistence.criteria.JoinType;
import javax.persistence.criteria.Predicate;
import javax.persistence.criteria.Root;
import javax.persistence.criteria.Subquery;

import org.springframework.data.jpa.domain.Specification;

import com.xmt.gameet.entities.Genre;
import com.xmt.gameet.entities.Genre_;
import com.xmt.gameet.entities.User;
import com.xmt.gameet.entities.User_;

public class GenreSpecifications {
	
	public static Specification<Genre> orderByName(){
		return new Specification<Genre>(){

			public Predicate toPredicate(Root<Genre> root, CriteriaQuery<?> query, CriteriaBuilder cb) {
				query.orderBy(cb.asc(root.get(Genre_.name)));
				return cb.conjunction();
			}
			
		};
	}
	
	
	public static Specification<Genre> addNameFilter(final String name){
		return new Specification<Genre>(){

			public Predicate toPredicate(Root<Genre> root, CriteriaQuery<?> query, CriteriaBuilder cb) {
				Expression<String> path = root.get("name");
				Expression<String> path2 = root.get("descriptionES");
				Predicate predicate = cb.or(cb.like(path, "%" + name + "%"),cb.like(path2, "%" + name + "%"));
				return predicate;
			}
			
		};
	}
	
	public static Specification<Genre> addAddedUserFilter(final Integer user){
		return new Specification<Genre>(){

			public Predicate toPredicate(Root<Genre> root, CriteriaQuery<?> query, CriteriaBuilder cb) {
				query.distinct(true);
				Predicate predicate = cb.conjunction();
	            Join<Genre, User> join = root.join(Genre_.users, JoinType.INNER);
	            predicate.getExpressions().add(join.get(User_.id).in(user));
	            return predicate;
			}
			
		};
	}
	
	public static Specification<Genre> addNotAddedUserFilter(final Integer user){
		return new Specification<Genre>(){

			public Predicate toPredicate(Root<Genre> root, CriteriaQuery<?> query, CriteriaBuilder cb) {
				query.distinct(true);
				
				Subquery<Integer> sq = query.subquery(Integer.class);
				Root<Genre> rootSQ = sq.from(Genre.class); 
				Join<Genre, User> joinSQ = rootSQ.join(Genre_.users, JoinType.INNER);
				sq.where(joinSQ.get(User_.id).in(user));
				sq.select(rootSQ.get(Genre_.id));
				sq.distinct(true);
				
				Predicate predicate = cb.conjunction();
				Join<Genre, User> join = root.join(Genre_.users, JoinType.LEFT);
	            predicate.getExpressions().add(cb.not(root.get(Genre_.id).in(sq)));
	            return predicate;
			}
			
		};
	}
	
}
