package com.xmt.gameet.specifications;

import javax.persistence.criteria.CriteriaBuilder;
import javax.persistence.criteria.CriteriaQuery;
import javax.persistence.criteria.Expression;
import javax.persistence.criteria.Join;
import javax.persistence.criteria.JoinType;
import javax.persistence.criteria.Predicate;
import javax.persistence.criteria.Root;
import javax.persistence.criteria.Subquery;

import org.springframework.data.jpa.domain.Specification;

import com.xmt.gameet.entities.Language;
import com.xmt.gameet.entities.Language_;
import com.xmt.gameet.entities.User;
import com.xmt.gameet.entities.User_;

public class LanguageSpecifications {
	
	
	public static Specification<Language> orderByName(){
		return new Specification<Language>(){

			public Predicate toPredicate(Root<Language> root, CriteriaQuery<?> query, CriteriaBuilder cb) {
				query.orderBy(cb.asc(root.get(Language_.name)));
				return cb.conjunction();
			}
			
		};
	}
	
	public static Specification<Language> addNameFilter(final String name){
		return new Specification<Language>(){

			public Predicate toPredicate(Root<Language> root, CriteriaQuery<?> query, CriteriaBuilder cb) {
				Expression<String> path = root.get("name");
				return cb.like(path, "%" + name + "%");
			}
			
		};
	}
	
	public static Specification<Language> addAddedUserFilter(final Integer user){
		return new Specification<Language>(){

			public Predicate toPredicate(Root<Language> root, CriteriaQuery<?> query, CriteriaBuilder cb) {
				query.distinct(true);
				
				Predicate predicate = cb.conjunction();
	            Join<Language, User> join = root.join(Language_.users, JoinType.INNER);
	            predicate.getExpressions().add(join.get(User_.id).in(user));
	            return predicate;
			}
			
		};
	}
	
	public static Specification<Language> addNotAddedUserFilter(final Integer user){
		return new Specification<Language>(){

			public Predicate toPredicate(Root<Language> root, CriteriaQuery<?> query, CriteriaBuilder cb) {
				query.distinct(true);
				
				Subquery<Integer> sq = query.subquery(Integer.class);
				Root<Language> rootSQ = sq.from(Language.class); 
				Join<Language, User> joinSQ = rootSQ.join(Language_.users, JoinType.INNER);
				sq.where(joinSQ.get(User_.id).in(user));
				sq.select(rootSQ.get(Language_.id));
				sq.distinct(true);
				
				Predicate predicate = cb.conjunction();
				Join<Language, User> join = root.join(Language_.users, JoinType.LEFT);
	            predicate.getExpressions().add(cb.not(root.get(Language_.id).in(user)));
	            return predicate;
			}
			
		};
	}
	
}
