package com.xmt.gameet.specifications;

import javax.persistence.criteria.CriteriaBuilder;
import javax.persistence.criteria.CriteriaQuery;
import javax.persistence.criteria.Expression;
import javax.persistence.criteria.Join;
import javax.persistence.criteria.JoinType;
import javax.persistence.criteria.Predicate;
import javax.persistence.criteria.Root;
import javax.persistence.criteria.Subquery;

import org.springframework.data.jpa.domain.Specification;

import com.xmt.gameet.entities.Platform;
import com.xmt.gameet.entities.Platform_;
import com.xmt.gameet.entities.User;
import com.xmt.gameet.entities.User_;

public class PlatformSpecifications {
	
	public static Specification<Platform> orderByName(){
		return new Specification<Platform>(){

			public Predicate toPredicate(Root<Platform> root, CriteriaQuery<?> query, CriteriaBuilder cb) {
				query.orderBy(cb.asc(root.get(Platform_.name)));
				return cb.conjunction();
			}
			
		};
	}
	
	public static Specification<Platform> addNameFilter(final String name){
		return new Specification<Platform>(){

			public Predicate toPredicate(Root<Platform> root, CriteriaQuery<?> query, CriteriaBuilder cb) {
				Expression<String> path = root.get("name");
				return cb.like(path, "%" + name + "%");
			}
			
		};
	}
	
	public static Specification<Platform> addAddedUserFilter(final Integer user){
		return new Specification<Platform>(){

			public Predicate toPredicate(Root<Platform> root, CriteriaQuery<?> query, CriteriaBuilder cb) {
				query.distinct(true);
				
				Predicate predicate = cb.conjunction();
	            Join<Platform, User> join = root.join(Platform_.users, JoinType.INNER);
	            predicate.getExpressions().add(join.get(User_.id).in(user));
	            return predicate;
			}
			
		};
	}
	
	public static Specification<Platform> addNotAddedUserFilter(final Integer user){
		return new Specification<Platform>(){

			public Predicate toPredicate(Root<Platform> root, CriteriaQuery<?> query, CriteriaBuilder cb) {
				query.distinct(true);
				
				Subquery<Integer> sq = query.subquery(Integer.class);
				Root<Platform> rootSQ = sq.from(Platform.class); 
				Join<Platform, User> joinSQ = rootSQ.join(Platform_.users, JoinType.INNER);
				sq.where(joinSQ.get(User_.id).in(user));
				sq.select(rootSQ.get(Platform_.id));
				sq.distinct(true);
				
				Predicate predicate = cb.conjunction();
				 Join<Platform, User> join = root.join(Platform_.users, JoinType.LEFT);
	            predicate.getExpressions().add(cb.not(root.get(Platform_.id).in(user)));
	            return predicate;
			}
			
		};
	}
	
}
