package com.xmt.gameet.specifications;

import java.util.ArrayList;
import java.util.List;

import javax.persistence.criteria.CriteriaBuilder;
import javax.persistence.criteria.CriteriaQuery;
import javax.persistence.criteria.Expression;
import javax.persistence.criteria.Join;
import javax.persistence.criteria.JoinType;
import javax.persistence.criteria.Predicate;
import javax.persistence.criteria.Root;
import javax.persistence.criteria.Subquery;

import org.springframework.data.jpa.domain.Specification;

import com.xmt.gameet.entities.Game;
import com.xmt.gameet.entities.Game_;
import com.xmt.gameet.entities.Genre;
import com.xmt.gameet.entities.Genre_;
import com.xmt.gameet.entities.Language;
import com.xmt.gameet.entities.Language_;
import com.xmt.gameet.entities.Platform;
import com.xmt.gameet.entities.Platform_;
import com.xmt.gameet.entities.Timetable;
import com.xmt.gameet.entities.Timetable_;
import com.xmt.gameet.entities.User;
import com.xmt.gameet.entities.UserRelationActual;
import com.xmt.gameet.entities.User_;
import com.xmt.gameet.helpers.Day;
import com.xmt.gameet.helpers.SearchFormModel;

public class UserSpecifications {

	
	public static Specification<User> addEmailFilter(final String email){
		return new Specification<User>(){

			public Predicate toPredicate(Root<User> root, CriteriaQuery<?> query, CriteriaBuilder cb) {
				Expression<String> path = root.get("email");
				return cb.equal(path, email);
			}
			
		};
	}
	
	public static Specification<User> addNicknameFilter(final String nickname){
		return new Specification<User>(){

			public Predicate toPredicate(Root<User> root, CriteriaQuery<?> query, CriteriaBuilder cb) {
				Expression<String> path = root.get("nickname");
				return cb.equal(path, nickname);
			}
			
		};
	}
	
	public static Specification<User> addUserRelation(final Integer user){
		return new Specification<User>(){

			public Predicate toPredicate(Root<User> root, CriteriaQuery<?> query, CriteriaBuilder cb) {
				
				Subquery<Integer> sq = query.subquery(Integer.class);
				Root<User> rootSQ = sq.from(User.class); 
				Join<User, UserRelationActual> joinSQ = rootSQ.join(User_.userRelations, JoinType.INNER);
				sq.where(joinSQ.get("user1").in(user));
				sq.select(rootSQ.get(User_.id));
				sq.distinct(true);
				
				Predicate predicate = cb.conjunction();
				Join<User, UserRelationActual> join = root.join(User_.userRelations, JoinType.LEFT);
	            predicate.getExpressions().add(root.get(User_.id).in(sq));
				
	            return predicate;
			}
			
		};
	}
	
	public static Specification<User> addSearchFilters(final SearchFormModel searchModel){
		return new Specification<User>(){

			public Predicate toPredicate(Root<User> root, CriteriaQuery<?> query, CriteriaBuilder cb) {
				Predicate predicate =  cb.conjunction();
				query.distinct(true);
				if(searchModel.getNickname() != null && !searchModel.getNickname().isEmpty()){
					predicate.getExpressions().add(cb.like(root.get(User_.nickname), "%" + searchModel.getNickname() + "%"));
				}
				if((searchModel.getAgeFrom() != null && searchModel.getAgeFrom() != 0 ) 
						&& (searchModel.getAgeTo() != null && searchModel.getAgeTo() != 0)){
					predicate.getExpressions().add(cb.between(root.get(User_.age), searchModel.getAgeFrom(), searchModel.getAgeTo()));
				}
				if(searchModel.getLanguages() != null && searchModel.getLanguages().size() > 0){
					Join<User,Language> join = root.join(User_.languages, JoinType.INNER);
					predicate.getExpressions().add(join.get(Language_.id).in(searchModel.getLanguages()));
				}
				if(searchModel.getGames() != null && searchModel.getGames().size() > 0){
					Join<User,Game> join = root.join(User_.gamesToPlay, JoinType.INNER);
					predicate.getExpressions().add(join.get(Game_.id).in(searchModel.getGames()));
				}
				if(searchModel.getPlatforms() != null && searchModel.getPlatforms().size() > 0){
					Join<User,Platform> join = root.join(User_.platformsPrefered, JoinType.INNER);
					predicate.getExpressions().add(join.get(Platform_.id).in(searchModel.getPlatforms()));
				}
				if(searchModel.getPlatform() != null && searchModel.getPlatform() != 0){
					Join<User,Platform> join = root.join(User_.platformsPrefered, JoinType.INNER);
					predicate.getExpressions().add(join.get(Platform_.id).in(searchModel.getPlatform()));
				}
				if(searchModel.getGenre() != null && searchModel.getGenre() != 0){
					Join<User,Genre> join = root.join(User_.genresPrefered, JoinType.INNER);
					predicate.getExpressions().add(join.get(Genre_.id).in(searchModel.getGenre()));
				}
				if(searchModel.getGenres() != null && searchModel.getGenres().size() > 0){
					Join<User,Genre> join = root.join(User_.genresPrefered, JoinType.INNER);
					predicate.getExpressions().add(join.get(Genre_.id).in(searchModel.getGenres()));
				}
				if(searchModel.getGame() != null && searchModel.getGame() != 0){
					Join<User,Game> join = root.join(User_.gamesToPlay, JoinType.INNER);
					predicate.getExpressions().add(join.get(Game_.id).in(searchModel.getGame()));
				}
				if(!searchModel.isAnyDay() && searchModel.isAnyTime()){
					List<String> daysSelected = new ArrayList<String>();
					for(Day day : searchModel.getDays()){
						daysSelected.add(day.getCode());
					}
					Join<User,Timetable> join = root.join(User_.timetable, JoinType.INNER);
					predicate.getExpressions().add(join.get(Timetable_.day).in(daysSelected));
				}
				if(!searchModel.isAnyDay() && !searchModel.isAnyTime()){
					List<String> daysSelected = new ArrayList<String>();
					for(Day day : searchModel.getDays()){
						daysSelected.add(day.getCode());
					}
					Join<User,Timetable> join = root.join(User_.timetable, JoinType.INNER);
					predicate.getExpressions().add(cb.and(join.get(Timetable_.day).in(daysSelected)));
					if(searchModel.getHourFrom() != null && searchModel.getHourFrom() != 0) {
						predicate.getExpressions().add(cb.greaterThanOrEqualTo(join.get(Timetable_.initialTime), searchModel.getHourFrom()));
					}
					if(searchModel.getHourTo() != null && searchModel.getHourFrom() != 0) {
						predicate.getExpressions().add(cb.lessThanOrEqualTo(join.get(Timetable_.endingTime), searchModel.getHourTo()));
					}
				}
				if(searchModel.isAnyDay() && !searchModel.isAnyTime()){
					if(searchModel.getHourFrom() != null && searchModel.getHourFrom() != 0) {
						Join<User,Timetable> join = root.join(User_.timetable, JoinType.INNER);
						predicate.getExpressions().add(cb.greaterThanOrEqualTo(join.get(Timetable_.initialTime), searchModel.getHourFrom()));
					}
					if(searchModel.getHourTo() != null && searchModel.getHourFrom() != 0) {
						Join<User,Timetable> join = root.join(User_.timetable, JoinType.INNER);
						predicate.getExpressions().add(cb.lessThanOrEqualTo(join.get(Timetable_.endingTime), searchModel.getHourTo()));
					}
				}
				predicate.getExpressions().add(cb.equal(root.get(User_.profileCompleted), 1));
				predicate.getExpressions().add(cb.not(cb.equal(root.get(User_.id), searchModel.getUserSearchingId())));
				query.orderBy(cb.asc(root.get(User_.recommendationLevel)));
				
				return predicate;
			}
			
		};
	}
	
	
	
	
}
